#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from PyQt6.QtWidgets import QLineEdit, QApplication, QLabel, QHBoxLayout, QVBoxLayout, QPushButton,\
                            QComboBox, QDialog, QListWidget, QAbstractScrollArea
from PyQt6.QtCore import Qt
from jhora import utils
from jhora.panchanga import drik, vratha
import datetime
_main_window_width = 550
_main_window_height = 550
_CALCULATE_WAIT_MSG = '<b>Calculating... Results will be updated here. Please wait...</b>'
class VrathaFinderDialog(QDialog):
    """
        Search Vratha Dialog
    """
    def __init__(self,current_date_jd,place:drik.Place,chart_title='Search for special days'):
        super().__init__()
        self._current_date_jd = current_date_jd
        self._selection_date_jd = current_date_jd
        self.Place = place
        self._chart_title = chart_title
        self._accept_clicked = False
        self.res = utils.resource_strings
        self._vratha_result_dates = []
        self._tithi_list = [self.res['tithi_str']]+ [utils.PAKSHA_LIST[0]+'-'+t for t in utils.TITHI_LIST[0:14]]+ \
                      [utils.TITHI_LIST[14]]+[utils.PAKSHA_LIST[1]+'-'+t for t in utils.TITHI_LIST[0:14]]+ \
                                   [utils.TITHI_LIST[29]]
        self._nakshathra_list = [self.res['nakshatra_str']]+utils.NAKSHATRA_LIST
        self._yogam_list = [self.res['yogam_str']]+utils.YOGAM_LIST
        self._tm_list = [self.res['tamil_month_str']]+utils.MONTH_LIST
        self._create_ui()
    def _create_ui(self):
        v_layout = QVBoxLayout()
        h_layout = QHBoxLayout()
        self._special_combo = QComboBox()
        self._special_combo.addItems([self.res[k+'_str'] for k in list(vratha.special_vratha_map.keys())+['customized']])
        self._special_combo.currentIndexChanged.connect(self._enable_combos)
        h_layout.addWidget(self._special_combo)
        self._from_date_label = QLabel(self.res['starts_at_str']); h_layout.addWidget(self._from_date_label)
        y,m,d,_ = utils.jd_to_gregorian(self._current_date_jd)
        self._from_date_text = QLineEdit(str(y)+','+str(m)+','+str(d));h_layout.addWidget(self._from_date_text)
        self._to_date_label = QLabel(self.res['ends_at_str']); h_layout.addWidget(self._to_date_label)
        y,m,d,_ = utils.jd_to_gregorian(self._current_date_jd+365)
        self._to_date_text = QLineEdit(str(y)+','+str(m)+','+str(d));h_layout.addWidget(self._to_date_text)
        v_layout.addLayout(h_layout)
        h_layout = QHBoxLayout()
        v_layout.addLayout(h_layout)
        h_layout = QHBoxLayout()
        self._tithi_combo = QComboBox()
        self._tithi_combo.addItems(self._tithi_list)
        h_layout.addWidget(self._tithi_combo)
        self._nakshatra_combo = QComboBox()
        self._nakshatra_combo.addItems(self._nakshathra_list)
        h_layout.addWidget(self._nakshatra_combo)
        self._yogam_combo = QComboBox()
        self._yogam_combo.addItems(self._yogam_list)
        h_layout.addWidget(self._yogam_combo)
        self._tm_combo = QComboBox()
        self._tm_combo.addItems(self._tm_list)
        h_layout.addWidget(self._tm_combo)
        v_layout.addLayout(h_layout)
        self._tithi_combo.setVisible(False); self._nakshatra_combo.setVisible(False)
        self._yogam_combo.setVisible(False); self._tm_combo.setVisible(False)
        self._tithi_combo.setEnabled(False); self._nakshatra_combo.setEnabled(False)
        self._yogam_combo.setEnabled(False); self._tm_combo.setEnabled(False)
        self._tithi_combo.currentIndexChanged.connect(self._enable_combos)
        self._nakshatra_combo.currentIndexChanged.connect(self._enable_combos)
        self._yogam_combo.currentIndexChanged.connect(self._enable_combos)
        self._tm_combo.currentIndexChanged.connect(self._enable_combos)
        self._results_list = QListWidget()
        #self._results_list.setEnabled(False)
        self._results_list.setSizeAdjustPolicy(QAbstractScrollArea.SizeAdjustPolicy.AdjustToContents)
        v_layout.addWidget(self._results_list)
        self._stats_label = QLabel('')
        v_layout.addWidget(self._stats_label)
        h_layout = QHBoxLayout()
        self._find_button = QPushButton(self.res['compute_str']); h_layout.addWidget(self._find_button)
        self._find_button.clicked.connect(self._calculate_clicked)
        self._accept_button = QPushButton(self.res['accept_str']); h_layout.addWidget(self._accept_button)
        self._accept_button.clicked.connect(self._accept_and_close)
        self._close_button = QPushButton(self.res['cancel_str']); h_layout.addWidget(self._close_button)
        self._close_button.clicked.connect(self._close_dialog)
        v_layout.addLayout(h_layout)
        self.setLayout(v_layout)
        self.setMinimumSize(_main_window_width,_main_window_height)
        self.setWindowTitle(self._chart_title)
        self._accept_button.setEnabled(False)
        self._calculate_clicked()
    def closeEvent(self, *args, **kwargs):
        self._selection_date_jd = self._current_date_jd
        QApplication.restoreOverrideCursor()
        return QDialog.closeEvent(self, *args, **kwargs)
    def _enable_combos(self):
        vrata_index = self._special_combo.currentIndex()
        self._results_list.clear();self._accept_button.setEnabled(False)
        if vrata_index == len(vratha.special_vratha_map.keys()):#vratha_type in ['tithi','nakshatra','yogam']:
            self._tithi_combo.setVisible(True); self._nakshatra_combo.setVisible(True)
            self._yogam_combo.setVisible(True); self._tm_combo.setVisible(True)
            self._tithi_combo.setEnabled(True); self._nakshatra_combo.setEnabled(True)
            self._yogam_combo.setEnabled(True); self._tm_combo.setEnabled(True)
        else:
            self._tithi_combo.setVisible(False); self._nakshatra_combo.setVisible(False)
            self._yogam_combo.setVisible(False); self._tm_combo.setVisible(False)
            self._tithi_combo.setEnabled(False); self._nakshatra_combo.setEnabled(False)
            self._yogam_combo.setEnabled(False); self._tm_combo.setEnabled(False)
        self._calculate_clicked()
    def _calculate_clicked(self):
        self._accept_button.setEnabled(False)
        _start_date = self._from_date_text.text().split(','); _start_date=drik.Date(int(_start_date[0]),int(_start_date[1]),int(_start_date[2]))
        _end_date = self._to_date_text.text().split(','); _end_date=drik.Date(int(_end_date[0]),int(_end_date[1]),int(_end_date[2]))
        vrata_index = self._special_combo.currentIndex()
        if vrata_index == len(vratha.special_vratha_map.keys()):#vratha_type in ['tithi','nakshatra','yogam']:
            _tithi_index = None if self._tithi_combo.currentIndex()==0 else self._tithi_combo.currentIndex()
            _nakshatra_index = None if self._nakshatra_combo.currentIndex()==0 else self._nakshatra_combo.currentIndex()
            _yogam_index = None if self._yogam_combo.currentIndex()==0 else self._yogam_combo.currentIndex()
            _tm_index = None if self._tm_combo.currentIndex()==0 else self._tm_combo.currentIndex()
            vdates = vratha.search(self.Place,_start_date, _end_date, _tithi_index, _nakshatra_index, _yogam_index, _tm_index)
        else:
            vratha_type = list(vratha.special_vratha_map.keys())[vrata_index]
            vdates = vratha.special_vratha_dates(self.Place, _start_date, _end_date, vratha_type)
        if len(vdates)>0:
            self._stats_label.setText(str(len(vdates))+' matching dates found')
            self._vratha_result_dates = vdates
            self._results_list.clear()
            for values in vdates:
                res = ''
                res += '-'.join(map(str,values[0]))
                if len(values)>1: res += ' / '+str(utils.to_dms(values[1]))
                if len(values)>2: res += ' / '+str(values[-1])
                self._results_list.addItem(res)
            self._results_list.setCurrentRow(0)
            self._accept_button.setEnabled(True)
        else:
            self._stats_label.setText('No matching dates found')
            self._results_list.clear()
    def _accept_and_close(self):
        selected_date_index = self._results_list.currentRow()
        #print('selected date index',selected_date_index)
        if selected_date_index !=-1:
            selected_item = self._vratha_result_dates[selected_date_index]
            #print('selected vdate',selected_item)
            selected_date = self._vratha_result_dates[selected_date_index][0]
            selected_time = self._vratha_result_dates[selected_date_index][1]
            self._selection_date_jd = utils.julian_day_number(selected_date, (0,0,0))
            #selected_time = drik.sunrise(self._selection_date_jd, self.Place)[0]
            self._selection_date_jd = utils.julian_day_number(selected_date, (selected_time,0,0))
            #print('select JD',self._selection_date_jd,selected_date,utils.to_dms(selected_time))
            y,m,d,fh = utils.jd_to_gregorian(self._selection_date_jd)
            #print(y,m,d,utils.to_dms(fh))
        self._accept_clicked = True
        self.accept()
    def _close_dialog(self):
        self._accept_clicked = False
        self.reject()
def show(current_date_jd,place:drik.Place,chart_title='Search for special days'):
    import sys
    def except_hook(cls, exception, traceback):
        sys.__excepthook__(cls, exception, traceback)
    sys.excepthook = except_hook
    dlg = VrathaFinderDialog(current_date_jd,place,chart_title)
    ret = dlg.exec()
    print('exec return value',ret)
    return ret
if __name__ == "__main__":
    import sys
    def except_hook(cls, exception, traceback):
        print('exception called')
        sys.__excepthook__(cls, exception, traceback)
    sys.excepthook = except_hook
    lang = 'ta'; utils.set_language(lang)
    App = QApplication(sys.argv)
    jd = utils.julian_day_number((2024,1,1), (10,34,0))
    place = drik.Place('Chennai,India',13.0878,80.2785,5.5)
    entry_type = 0
    show(jd, place)
    exit()
    chart = VrathaFinderDialog(jd,place)
    chart.show()
    sys.exit(App.exec())




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
""" To Calculate strengths of planets/rasis from chart positions of planets """
""" Ref: https://www.scribd.com/document/426763000/Shadbala-and-Bhavabala-Calculation-pdf """
""" Ref: https://medium.com/thoughts-on-jyotish/shadbala-the-6-sources-of-strength-4c5befc0c59a """
from jhora import const,utils
from jhora.panchanga import drik
from jhora.horoscope.chart import charts, house

kendras = lambda asc_house:[(asc_house+h-1)%12 for h in [1,4,7,10] ]
panapharas = lambda asc_house:[(asc_house+h-1)%12 for h in [2,5,8,11] ]
apoklimas = lambda asc_house:[(asc_house+h-1)%12 for h in [3,6,9,12] ]

def harsha_bala(dob,tob,place,divisional_factor=1):
    """
        computes the harsha bala score of the planets
        @param dob: date of solar entry in the annual chart / date of birth in natal chart
        @param tob: time of solar entry in the annual chart / time of birth in natal chart
        @return: Harsha Bala score for each planet - as a list 
            Example: {0: 0, 1: 15, 2: 0, 3: 10, 4: 5, 5: 10, 6: 5} - Sun's score = 0, Venus's score = 10
    """
    jd = utils.julian_day_number(dob, tob)
    sun_rise = drik.sunrise(jd, place)[0]
    sun_set = drik.sunset(jd, place)[0]
    new_year_daytime_start = True
    fh = utils.from_dms(tob[0],tob[1],tob[2])
    if fh < sun_rise or fh > sun_set:
        new_year_daytime_start = False
    planet_positions = charts.divisional_chart(jd, place,divisional_chart_factor=divisional_factor)
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    asc_house = p_to_h[const._ascendant_symbol]
    harsha_bala = {p:0 for p in range(7) }
    for p in range(7):
        h_p = p_to_h[p]
        h_f_a = (p_to_h[p]-asc_house)%12
        " Rule-1 - planets in their harsha bala houses"
        if const.harsha_bala_houses[p] == h_f_a:
            harsha_bala[p] +=5
        " Rule-2 - exhalted planets in their own house "
        if const.house_strengths_of_planets[p][h_p] > const._FRIEND or h_p in const.house_lords_dict[p]: # Exhalted or Own
            harsha_bala[p]+= 5
        " Rule-3 Feminine"
        if p in const.feminine_planets and h_f_a in const.harsha_bala_feminine_houses:
            harsha_bala[p] += 5
        elif p in const.masculine_planets and h_f_a in const.harsha_bala_masculine_houses:
            harsha_bala[p] += 5
        "Rule-4 "
        if new_year_daytime_start and p in const.masculine_planets:
            harsha_bala[p] += 5
        elif not new_year_daytime_start and p in const.feminine_planets:
            harsha_bala[p] += 5
    return harsha_bala
""" 
    Ref: https://www.scribd.com/document/426763000/Shadbala-and-Bhavabala-Calculation-pdf
"""    
def _kshetra_bala(p_to_h_of_rasi_chart):
    kb = {p:0 for p in range(7) }
    for p in range(7):
        h_p = p_to_h_of_rasi_chart[p]
        if const.house_strengths_of_planets[p][h_p] > const._FRIEND:
            kb[p] = 30
        elif const.house_strengths_of_planets[p][h_p] == const._FRIEND:
            kb[p] = 15 #22.5
        elif const.house_strengths_of_planets[p][h_p] == const._ENEMY:#const._DEFIBILATED_NEECHAM:
            kb[p] = 7.5
    return kb.values()
def _sapthavargaja_bala(jd,place):
    sv = [1, 2, 3, 7, 9, 12, 30]
    pp_sv = {}
    for dcf in sv:
        pp = charts.divisional_chart(jd, place,divisional_chart_factor=dcf)
        pp_sv[dcf] = pp
    svb = []
    for dcf in sv:
        svbc = _sapthavargaja_bala_1(pp_sv[dcf],dcf)
        svb.append(svbc)
    svb_sum = list(map(sum,zip(*svb)))
    return svb_sum
def _sthana_bala(jd, place):
    sv = [1, 2, 3, 7, 9, 12, 30]
    pp_sv = {}
    for dcf in sv:
        pp = charts.divisional_chart(jd, place,divisional_chart_factor=dcf)
        pp_sv[dcf] = pp
    ub = _uchcha_bala(pp_sv[1])
    svb = _sapthavargaja_bala(jd, place)
    ob = _ojayugama_bala(pp_sv[1], pp_sv[9])
    kb = _kendra_bala(pp_sv[1])
    db = _dreshkon_bala(pp_sv[1])
    sb = list(map(sum,zip(*[ub,svb,ob,kb,db])))
    sb = [round(v,2) for v in sb]
    return sb
def _sapthavargaja_bala_1(planet_positions,dcf):
    sb = [0 for _ in range(8)]
    #sb_fac = {const._ADHISATHRU_GREATENEMY:4,const._SATHRU_ENEMY:4,const._SAMAM_NEUTRAL:10,const._MITHRA_FRIEND:15,
    #          const._ADHIMITRA_GREATFRIEND:20}
    sb_fac = {const._ADHISATHRU_GREATENEMY:1.875,const._SATHRU_ENEMY:3.75,const._SAMAM_NEUTRAL:7.5,const._MITHRA_FRIEND:15,
              const._ADHIMITRA_GREATFRIEND:22.5}
    for p,(h,_) in planet_positions[1:8]:
        owner = house.house_owner_from_planet_positions(planet_positions, h)
        if h == const.moola_trikona_of_planets[p] and dcf==1: # Moola Trinkona Rasi
            sb[p] = 45
        elif const.house_strengths_of_planets[p][h]==const._OWNER_RULER: # Swastha Rasi
            sb[p] = 30
        else:
            sb[p] = sb_fac[const.compound_planet_relations[p][owner]]
    return sb
def _ojayugama_bala(rasi_planet_positions, navamsa_planet_positions):
    sb = [0 for _ in range(7)]
    for p in range(7):
        rh = rasi_planet_positions[p+1][1][0]
        nh = navamsa_planet_positions[p+1][1][0]
        if p in [1,5]:
            if rh in const.even_signs:
                sb[p] = 15
            if nh in const.even_signs:
                sb[p] += 15
        else:
            if rh in const.odd_signs:
                sb[p] = 15
            if nh in const.odd_signs:
                sb[p] += 15
    return sb
def _kendra_bala(rasi_planet_positions):
    kb = [0 for _ in range(7)]
    asc_house = rasi_planet_positions[0][1][0]
    for p,(h,_) in rasi_planet_positions[1:8]: #exclude 0th element Lagnam and Rahu/Ketu
        if h in kendras(asc_house):
            kb[p] = 60
        elif h in panapharas(asc_house):
            kb[p] = 30
        elif h in apoklimas(asc_house):
            kb[p] = 15
    return kb
def _dreshkon_bala(planet_positions):
    kb = [0 for _ in range(7)]
    kbf = [(0,2,4),(3,6),(1,5)]
    for p,(h,long) in planet_positions[1:8]: #exclude 0th element Lagnam and Rahu/Ketu
        pd = int(long//10.0)
        if p in kbf[pd]:
            kb[p] = 15
    return kb
def _uccha_rashmi(planet_positions):
    ub = []
    for p,(h,long) in planet_positions[1:const._pp_count_upto_saturn]: #exclude 0th element Lagnam and Rahu/Ketu
        p_long = h*30+long
        pd = (const.planet_deep_debilitation_longitudes[p]+360 - p_long)%360
        if pd > 180.0:
            pd = 360.0 - pd
        pd += 30
        ub.append(round(pd*2/30,1))
    return ub    
def _uchcha_bala(planet_positions):
    ub = []
    for p,(h,long) in planet_positions[1:const._pp_count_upto_saturn]: #exclude 0th element Lagnam and Rahu/Ketu
        p_long = h*30+long
        pd = (p_long+360 - const.planet_deep_debilitation_longitudes[p])%360
        if pd > 180.0:
            pd = 360.0 - pd
        if const.use_saravali_formula_for_uccha_bala:
            ubv = round(pd/3,2)
            ub.append(ubv) # Saravali formula #https://saravali.github.io/astrology/bala_sthana.html#uchcha
        else: # PVR Book formula
            ubv = round(pd/180.0*20.0,2)
            ub.append(ubv) # PVR formula
    return ub
def __hadda_points(rasi,p_long,p):
    l_range = const.hadda_lords[rasi]
    hp = [planet for planet,long in l_range if p_long<=long ][0]
    if p == hp:
        return const.hadda_points[0]
    elif hp in const.friendly_planets[p]:
        return const.hadda_points[1]
    elif hp in const.enemy_planets[p]:
        return const.hadda_points[2]
    return 0.0
def _hadda_bala(planet_positions):
    hb = [ __hadda_points(h, p_long,p) for p,(h,p_long) in planet_positions[1:8]]
    return hb
def _drekkana_bala(p_to_h_of_drekkana_chart):
    kb = {p:0 for p in range(7) }
    for p in range(7):
        h_p = p_to_h_of_drekkana_chart[p]
        if const.house_strengths_of_planets[p][h_p] > const._FRIEND:
            kb[p] = 10
        elif const.house_strengths_of_planets[p][h_p]==const._FRIEND:
            kb[p] = 5
        elif const.house_strengths_of_planets[p][h_p]==const._ENEMY:
            kb[p] = 2.5
    return kb
def _navamsa_bala(p_to_h_navamsa_chart):
    kb = {p:0 for p in range(7) }
    for p in range(7):
        h_p = p_to_h_navamsa_chart[p]
        if const.house_strengths_of_planets[p][h_p]>const._FRIEND:
            kb[p] = 5
        elif const.house_strengths_of_planets[p][h_p]==const._FRIEND:
            kb[p] = 2.5
        elif const.house_strengths_of_planets[p][h_p]==const._ENEMY:
            kb[p] = 1.25
    return kb
def pancha_vargeeya_bala(jd,place):
    """
        computes the Pancha Vargeeya bala score of the planets
            Keshetra Bala:
                A planet gets 30 units of Bala in own sign, 22.5 units in friendly sign, 15 units in neutral sign 
                and 7.5 units in an enemy sign.
            Drekkana Bala
                A planet in own rasi in D-3 gets 10 units of Drekkana bala. A planet in a friend’s rasi in D-3 gets 5 units of
                Drekkana bala. A planet in an enemy’s rasi in D-3 gets 2.5 units of Drekkana bala.
            Navamsa Bala
                A planet in own rasi in D-9 gets 5 units of Navamsa bala. A planet in a friend’s rasi in D-9 gets 2.5 units of
                Navamsa bala. A planet in an enemy’s rasi in D-9 gets 1.25 units of Navamsa bala.
            Uchcha Bala
                Uchcha bala shows how close a planet is from its exaltation point. A planet gets 20
                units of uchcha bala if it is at its deep exaltation point (Sun: 10º Ar, Moon: 3º Ta,
                Mars: 28º Cp, Mercury: 15º Vi, Jupiter: 5º Cn, Venus: 27º Pi, Saturn: 20º Li). At
                180º from its deep exaltation point, a planet is deeply debilitated and it gets 0 units of
                uchcha bala.
            Hadda Bala
                A planet in own hadda gets 15 units of Hadda bala. A planet in a friend’s hadda gets 7.5 units of Hadda bala. 
                A planet in an enemy’s hadda gets 3.75 units of Hadda bala.
        @param jd: Julian Day Number (of the annual day
        @param place: drik.Place struct: Place('place_name',latitude, longitude, timezone) 
        @return: Pancha Vargeeya Bala score for each planet - as a list 
            Example: [15.72, 14.27, 13.0, 6.33, 11.87, 16.05, 6.45] - Sun's score = 15.72, Venus's score = 16.05
    """
    rasi_chart = charts.divisional_chart(jd, place, divisional_chart_factor=1)
    p_to_h_of_rasi_chart = utils.get_planet_house_dictionary_from_planet_positions(rasi_chart)
    kb = _kshetra_bala(p_to_h_of_rasi_chart)
    ub = _uchcha_bala(rasi_chart)
    hb = _hadda_bala(rasi_chart)
    drekkana_chart = charts.divisional_chart(jd, place,divisional_chart_factor=3)
    p_to_h_of_drekkana_chart = utils.get_planet_house_dictionary_from_planet_positions(drekkana_chart)
    db = _drekkana_bala(p_to_h_of_drekkana_chart)
    navamsa_chart = charts.divisional_chart(jd, place,divisional_chart_factor=9)
    p_to_h_of_navamsa_chart = utils.get_planet_house_dictionary_from_planet_positions(navamsa_chart)
    nb = _navamsa_bala(p_to_h_of_navamsa_chart)
    pvb = [kb,ub,hb,db,nb]
    pvb = [round(sum(x)/4.0,2) for x in zip(*pvb)]
    pvbd = {k:pvb[k] for k in range(7)}
    return pvbd
def dwadhasa_vargeeya_bala(jd,place):
    """
        Calculates dwadhasa_vargeeya_bala score of the planets
        @param jd: Julian Day Number (of the annual day
        @param place: drik.Place struct: Place('place_name',latitude, longitude, timezone) 
        @return:   returns dict of strong (>0) and weak (<0) planets. Also returns list of only strong planets
            Example: {0: -4, 1: 0, 2: -4, 3: 2, 4: 0, 5: -2, 6: 2} [3, 6]
    """
    dvp = {p:0 for p in range(7) }
    for dvf in range(1,13): #D1-D12 charts
        planet_positions = charts.divisional_chart(jd, place, divisional_chart_factor=dvf)
        p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
        for p in range(7):
            if const.house_strengths_of_planets[p][p_to_h[p]] >= const._FRIEND:
                dvp[p]+=1
    dvpd = {k:dvp[k] for k in range(7)}
    return dvpd
def _dig_bala(jd,place):
    planet_positions = charts.rasi_chart(jd, place)
    powerless_houses_of_planets = [3,9,3,6,6,9,0]#[4,10,4,7,7,10,1]
    bm = drik.bhaava_madhya(jd, place)
    dbf = [bm[p] for p in powerless_houses_of_planets]
    dbp = [0 for _ in range(7)]
    for p,(h,long) in planet_positions[1:7]:
        p_long = h*30+long
        dbp[p] = round(abs(dbf[p]-p_long)/3,2)
    return dbp
def _divaratri_bala(jd,place):
    return _nathonnath_bala(jd,place)
def _nathonnath_bala(jd,place):
    nbp = [0 for _ in range(7)]
    _,_,_,tobh = utils.jd_to_gregorian(jd)
    mnhl = drik.midnight(jd, place)
    t_diff = (tobh - mnhl)*60/12
    for p in [0,4,5]:
        nbp[p] = round(t_diff,2)
    for p in [1,2,6]:
        nbp[p] = round(60 - t_diff,2)
    nbp[3] = 60.0
    return nbp
def _paksha_bala(jd,place):
    planet_positions = drik.dhasavarga(jd, place, divisional_chart_factor=1)
    sun_long = planet_positions[0][1][0]*30+planet_positions[0][1][1]
    moon_long = planet_positions[1][1][0]*30+planet_positions[1][1][1]
    pb = round(abs(sun_long - moon_long) / 3.0,2)
    pbp = [pb for _ in range(7)]
    cht_benefics = charts.benefics(jd, place)
    for p in cht_benefics:# const.natural_benefics:
        pbp[p] = pb
    for p in const.natural_malefics[:-2]: #Exc lude Rahu Kethu
        pbp[p] = round(60.0 - pb,2)
    pbp[1] *=2 
    return pbp
def _tribhaga_bala(jd,place):
    tbp = [0 for _ in range(7)]
    _,_,_,tobh = utils.jd_to_gregorian(jd)
    srh = drik.sunrise(jd, place)[0]
    ssh = drik.sunset(jd, place)[0]
    dl = drik.day_length(jd, place)
    nl = drik.night_length(jd, place)
    dlinc = dl/3 ; nlinc = nl / 3
    tbp[4] = 60
    if tobh > 0 and tobh < srh-nlinc:
        tbp[5] = 60
    elif tobh > srh-nlinc and tobh < srh:
        tbp[2] = 60
    elif tobh > srh and tobh < srh+dlinc:  
        tbp[3] = 60
    elif tobh > srh+dlinc and tobh < srh+2*dlinc:
        tbp[0] = 60
    elif tobh > srh+2*dlinc and tobh < ssh:
        tbp[5] = 60
    elif tobh > ssh and tobh < 24:
        tbp[1] = 60
    return tbp
def _abda_bala(jd,place):
    abp = [0 for _ in range(7)]
    day = drik.vaara(jd)
    abp[day] = 15
    return abp
def _masa_bala(jd,place):
    abp = [0 for _ in range(7)]
    day = drik.vaara(jd)
    abp[day] = 30
    return abp
def _vaara_bala(jd,place):
    abp = [0 for _ in range(7)]
    day = drik.vaara(jd)
    _,_,_,tobh = utils.jd_to_gregorian(jd)
    srise = drik.sunrise(jd, place)[0]
    if tobh < srise:
        day = (day-1)%7
    abp[day] = 45
    return abp
def _hora_bala(jd,place):
    abp = [0 for _ in range(7)]
    day = drik.vaara(jd)
    _,_,_,tobh = utils.jd_to_gregorian(jd)
    srise = drik.sunrise(jd, place)[0]
    if tobh < srise:
        day = (day-1)%7
        tobh += 24.0
    hora_order = [6,4,2,0,5,3,1]
    hora = (int(tobh-srise)+day+1)%7
    abp[hora_order[hora]] = 60
    return abp
def _ayana_bala(jd,place):
    declinations = drik.declination_of_planets(jd, place)
    ab = [0 for _ in range(7)]
    for p in range(7):
        ab[p] = round((24.0 + declinations[p])*1.25,2)
        if p==0:
            ab[p] *= 2
    return ab
def _yuddha_bala(jd,place):
    yb = [0 for _ in range(7)]
    pp = drik.dhasavarga(jd, place, divisional_chart_factor=1)[:7]
    p_longs = [h*30+long for _,(h,long) in pp]
    p_longs_copy = p_longs[:]
    ce = sorted(utils.closest_elements(p_longs, p_longs))
    indices = [p_longs.index(v) for v in ce]
    if any([sm==i for sm in [0,1] for i in indices]):
        return yb # All Zero
    # Find Sum of balas upto hora bala
    sb = _sthana_bala(jd, place)
    dgb = _dig_bala(jd,place)
    nb = _nathonnath_bala(jd, place)
    pb = _paksha_bala(jd,place)
    tb = _tribhaga_bala(jd, place)
    hb = _hora_bala(jd, place)
    bala_totals = [0 for _ in range(7)]
    for i in indices:
        bala_totals[i] += sb[i]
        bala_totals[i] += dgb[i]
        bala_totals[i] += nb[i]
        bala_totals[i] += pb[i]
        bala_totals[i] += tb[i]
        bala_totals[i] += hb[i]
    b_diff = abs(bala_totals[indices[0]] - bala_totals[indices[1]])
    dia_diff = abs(const.planets_disc_diameters[indices[0]]-const.planets_disc_diameters[indices[1]])
    y_bala = round(b_diff/dia_diff,2)
    yb[indices[0]] =  y_bala ; yb[indices[1]] =  -y_bala
    return yb
def _kaala_bala(jd,place):
    kb = [0 for _ in range(7)]
    nb = _nathonnath_bala(jd, place)
    pb = _paksha_bala(jd, place)
    tb = _tribhaga_bala(jd, place)
    ab = _abda_bala(jd, place)
    mb = _masa_bala(jd, place)
    vb = _vaara_bala(jd, place)
    hb = _hora_bala(jd, place)
    ayb = _ayana_bala(jd, place)
    yb = _yuddha_bala(jd, place)
    for p in range(7):
        kb[p] += nb[p]
        kb[p] += pb[p]
        kb[p] += tb[p]
        kb[p] += ab[p]
        kb[p] += mb[p]
        kb[p] += vb[p]
        kb[p] += hb[p]
        kb[p] += ayb[p]
        kb[p] += yb[p]
    kb = [round(kbp,2) for kbp in kb]
    return kb
def _ishta_phala(jd,place):
    planet_positions = charts.rasi_chart(jd, place)
    ip_score = {const._ADHIMITRA_GREATFRIEND:22,
                const._MITHRA_FRIEND:15,const._SAMAM_NEUTRAL:8,const._ADHISATHRU_GREATENEMY:4,
                const._SATHRU_ENEMY:2}
    ip = [0 for _ in range(7)]
    for p,(h,_) in planet_positions[1:const._pp_count_upto_saturn]:
        owner = house.house_owner_from_planet_positions(planet_positions, h)
        if const.house_strengths_of_planets[p][h]==const._EXALTED_UCCHAM:
            ip[p] = 60
        elif h == const.moola_trikona_of_planets[p]: # Moola Trinkona Rasi
            ip[p] = 45
        elif const.house_strengths_of_planets[p][h]==const._OWNER_RULER: # Swastha Rasi
            ip[p] = 30
        else:
            ip[p] = ip_score[const.compound_planet_relations[p][owner]]
    return ip
def _subha_rashmi(jd,place):
    planet_positions = charts.rasi_chart(jd, place)
    cr = _cheshta_rashmi(jd, place); ur = _uccha_rashmi(planet_positions)
    return [0.25*(cr[i]+ur[i]) for i in range(7)]
def _cheshta_rashmi(jd,place):
    """ STILL UNDER EXPERIMENT - Exact Algorithm unknown"""
    cb = [c*3.0 for c in _cheshta_bala(jd, place)]
    pp = drik.dhasavarga(jd, place, divisional_chart_factor=1)
    sun_long = pp[0][1][0]*30+pp[0][1][1]; moon_long = pp[1][1][0]*30+pp[1][1][1]
    cb[0] = (sun_long+90.0)%360 # Add 3 rasis to sun long
    cb[1] = (moon_long-sun_long)%360
    for p in range(7):
        if cb[p] > 180.0: cb[p] = 360.0 - cb[p]
        cb[p] = ((cb[p]+30)*2)/30
    return cb
def _cheshta_bala(jd,place):
    pp = drik.dhasavarga(jd, place, divisional_chart_factor=1)
    cb = [0 for _ in range(7)]
    from jhora.panchanga import surya_sidhantha
    sun_mean_long = surya_sidhantha._planet_mean_longitude(jd, place, const._SUN)
    for p in [const._MARS, const._MERCURY, const._JUPITER, const._VENUS, const._SATURN]: #range(2,7):
        p_id = drik.planet_list.index(p)
        mean_long = surya_sidhantha._planet_mean_longitude(jd, place, p)
        seegrocha = sun_mean_long
        if p in [const._MERCURY,const._VENUS]:
            seegrocha = mean_long
            mean_long = sun_mean_long
        true_long = pp[p_id][1][0]*30+pp[p_id][1][1]
        ave_long = 0.5*(true_long+mean_long)
        reduced_chesta_kendra = abs(seegrocha - ave_long)
        cb[p_id] = round(reduced_chesta_kendra/3,2)
    return cb
def _naisargika_bala(jd=None,place=None):
    return const.naisargika_bala[:-2]
def __drik_bala_calc_1(dk_p1_p2,p1,p2):
    _DEBUG_ = False
    """ 
        TODO: Aspects for div charts go beyond 60 (>100%) WHY???
        Should we use aspect strength factors 1/4, 1/2 3/4 and 1.0??
    """
    
    if _DEBUG_:  print(p1,p2,'drishti angle',dk_p1_p2)
    dk_p1_p2_new = dk_p1_p2
    if dk_p1_p2 >= 0 and dk_p1_p2 <= 30:
        dk_p1_p2_new = 0.0
        if _DEBUG_:  print('diff between',0,30,dk_p1_p2_new)
    elif dk_p1_p2 >= 30 and dk_p1_p2 <= 60:
        dk_p1_p2_new = 0.5*(dk_p1_p2-30.0)
        if _DEBUG_:  print('diff between',30,60,dk_p1_p2_new)
    elif dk_p1_p2 >= 60 and dk_p1_p2 <= 90:
        dk_p1_p2_new = (dk_p1_p2-60.0)+15
        if _DEBUG_:  print('diff between',60,90,dk_p1_p2_new)
        if p1 == 6: # Saturn
            dk_p1_p2_new += 45
            if _DEBUG_:  print('p1',6,'add',45,dk_p1_p2_new)
    elif dk_p1_p2 >= 90 and dk_p1_p2 <= 120:
        dk_p1_p2_new = 0.5*(120.0 - dk_p1_p2) + 30
        if _DEBUG_:  print('diff between',90,120,dk_p1_p2_new)
        if p1 == 2: # Mars
            dk_p1_p2_new += 15
            if _DEBUG_:  print('p1',2,'add',15,dk_p1_p2_new)
    elif dk_p1_p2 >= 120 and dk_p1_p2 <= 150:
        dk_p1_p2_new = (150.0 - dk_p1_p2)
        if _DEBUG_:  print('diff between',120,150,dk_p1_p2_new)
        if p1 == 4: # Jupiter
            dk_p1_p2_new += 30
            if _DEBUG_:  print('p1',4,'add',30,dk_p1_p2_new)
    elif dk_p1_p2 >= 150 and dk_p1_p2 <= 180:
        dk_p1_p2_new = 2.0*(dk_p1_p2 - 150)
        if _DEBUG_:  print('diff between',150,180,dk_p1_p2_new)
    elif dk_p1_p2 >= 180 and dk_p1_p2 <= 300:
        dk_p1_p2_new = 0.5*(300.0 - dk_p1_p2)
        if _DEBUG_:  print('diff between',180,300,dk_p1_p2_new)
        if p1 == 2 and (dk_p1_p2 >= 210 and dk_p1_p2 <= 240) : # Mars
            dk_p1_p2_new += 15
            if _DEBUG_:  print('p1',2,'add',15,dk_p1_p2_new)
        if p1 == 4 and (dk_p1_p2 >= 240 and dk_p1_p2 <= 270) : # Jupiter
            dk_p1_p2_new += 30
            if _DEBUG_:  print('p1',4,'add',30,dk_p1_p2_new)
        if p1 == 6 and (dk_p1_p2 >= 270 and dk_p1_p2 <= 300) : # Saturn
            dk_p1_p2_new += 45
            if _DEBUG_:  print('p1',6,'add',45,dk_p1_p2_new)
    else:
        dk_p1_p2_new = 0.0
        if _DEBUG_: print('<30 or >300',dk_p1_p2_new)
    dk_p1_p2_new = min(100,round(dk_p1_p2_new/60*100)) ## Forcing >100 to 100 Not sure this is correct?
    if _DEBUG_: print('final',p1,p2,'drishti value',dk_p1_p2_new)
    return dk_p1_p2_new
def planet_aspect_relationship_table(planet_positions,include_houses=False):
    _DEBUG_ = False
    pp = planet_positions[1:]
    rows = 21 if include_houses else 9
    dk = [[ 0 for _ in range(9)] for _ in range(rows)]
    for p1 in range(9): # Aspected Planet
        p1_long = pp[p1][1][0]*30+pp[p1][1][1]
        for p2 in range(9): # Aspecting Planet
            p2_long = pp[p2][1][0]*30+pp[p2][1][1]
            dk_p1_p2 = round((360.0+p1_long-p2_long)%360,2)
            if _DEBUG_: print('drishti angle',p2,p2_long,p1,p1_long,dk_p1_p2)
            dk_p1_p2 = __drik_bala_calc_1(dk_p1_p2,p2,p1)
            dk[p1][p2] = round(dk_p1_p2,2)
    if include_houses:
        asc_house = pp[0][1][0]; asc_long = pp[0][1][1]
        for h in range(12): # Aspected Planet
            h1 = (asc_house+h)%12
            p1_long = h1*30+asc_long
            for p2 in range(9): # Aspecting Planet
                p2_long = pp[p2][1][0]*30+pp[p2][1][1]
                dk_p1_p2 = round((360.0+p1_long-p2_long)%360,2)
                if _DEBUG_: print('drishti angle',p2,p2_long,p1,p1_long,dk_p1_p2)
                dk_p1_p2 = __drik_bala_calc_1(dk_p1_p2,p2,p1)
                dk[p1+h+1][p2] = round(dk_p1_p2,2)
    import numpy as np
    dk = np.array(dk).T
    return dk.tolist()
def _drik_bala(jd,place):
    dk = [[ 0 for _ in range(7)] for _ in range(7)]
    pp = charts.rasi_chart(jd, place)
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(pp)
    pp = pp[1:-2]
    subha_grahas = [1,3,4,5] ; asubha_grahas = [0,2,6]
    """ 
        TODO: Find out the aspect dictionary as below from the chart
        Following implementation of using rasi/graha drishti does not match with book values
        Should we use tajaka aspects? To be determined sometime in future version
        For example Sun aspects Moon and Venus, Moon aspects All but itelf...
        How to find this? Graha/Rasi Drishti?
    """ 
    _,_,gpp = house.graha_drishti_from_chart(house_planet_dict)
    _,_,rpp = house.raasi_drishti_from_chart(house_planet_dict)
    planet_aspects = {}
    for planet in range(7):
        planet_aspects[planet] = sorted(list(set(gpp[planet]+rpp[planet])))
        planet_aspects[planet] = [int(p) for p in planet_aspects[planet] if p not in [const._ascendant_symbol,'7','8']]
    for p1 in range(7): # Aspected Planet
        p1_long = pp[p1][1][0]*30+pp[p1][1][1]
        for p2 in range(7): # Aspecting Planet
            if p1 in planet_aspects[p2]:
                p2_long = pp[p2][1][0]*30+pp[p2][1][1]
                dk_p1_p2 = round((360.0+p1_long-p2_long)%360,2)
                dk_p1_p2 = __drik_bala_calc_1(dk_p1_p2,p2,p1)
            else:
                dk_p1_p2 = 0.0
            dk[p1][p2] = round(dk_p1_p2,2)
    import numpy as np
    dk = np.array(dk).T
    dkp = [0 for _ in range(7)] ; dkm = [0 for _ in range(7)]; dk_final = [0 for _ in range(7)]
    for row in range(7):
        for col in range(7):
            if row in subha_grahas:
                dkp[col] += dk[row][col] 
            if row in asubha_grahas:
                dkm[col] += dk[row][col]
            dk_final[col] = round((dkp[col] - dkm[col])/4,2) 
    return dk_final
def shad_bala(jd,place):
    sb = []
    stb = _sthana_bala(jd, place)
    sb.append(stb)
    kb = _kaala_bala(jd, place)
    sb.append(kb)
    dgb = _dig_bala(jd, place)
    sb.append(dgb)
    cb = _cheshta_bala(jd, place)
    sb.append(cb)
    nb = _naisargika_bala(jd, place)
    sb.append(nb)
    dkb = _drik_bala(jd, place)
    sb.append(dkb)
    import numpy as np
    sbn = np.array(sb)
    sb_sum = list(np.around(np.sum(sbn,0),2))
    sb_rupa = [round(ss/60.0,2) for ss in sb_sum]
    sb_req = [5,6,5,7,6.5,5.5,5]
    sb_strength = [round(sb_rupa[p]/sb_req[p],2) for p in range(7)]
    return [stb, kb, dgb, cb, nb, dkb, sb_sum, sb_rupa,sb_strength]
def _bhava_adhipathi_bala(jd,place):
    bhava_pp = charts.bhava_chart_houses(jd, place)
    asc_rasi = bhava_pp[const._ascendant_symbol][0]
    bb = []
    sb_sum = shad_bala(jd, place)[6]
    for h in range(12):
        r = (h+asc_rasi)%12
        owner = const.house_owners[r]
        bb.append(sb_sum[owner])
    return bb
def _bhava_dig_bala(jd,place):
    bdb = [0 for _ in range(12)]
    bm = drik.bhaava_madhya(jd, place)
    brl = {0:const.nara_rasi_longitudes,3:const.jalachara_rasi_longitudes,9:const.chatushpada_rasis,6:const.keeta_rasis}
    chk = []
    for k,v in brl.items():
        chk += list(set([((k+h)%12,abs(60-abs(h)*10)) for h in range(-7,7) for l1,l2 in v if bm[(k+h)%12] >= l1 and bm[(k+h)%12] <= l2]))
    chk = {k:v for k,v in chk}
    return list(dict(sorted(chk.items())).values())
def __bhava_drik_bala_calc_1(dk_p1_p2,p1):
    dk_p1_p2_new = dk_p1_p2
    if dk_p1_p2 > 0 and dk_p1_p2 <= 30.0:
        dk_p1_p2_new = 0.0
    elif dk_p1_p2 >= 30.01 and dk_p1_p2 <= 60.0:
        dk_p1_p2_new = 0.5*(dk_p1_p2-30.0)
    elif dk_p1_p2 >= 60.01 and dk_p1_p2 <= 90.0:
        dk_p1_p2_new = (dk_p1_p2-60.0)+15
        if p1 == 6: # Mars
            dk_p1_p2_new += 45
    elif dk_p1_p2 >= 90.01 and dk_p1_p2 <= 120.0:
        dk_p1_p2_new = 0.5*(120.0 - dk_p1_p2) + 30
        if p1 == 2: # Mars
            dk_p1_p2_new += 15
    elif dk_p1_p2 >= 120.01 and dk_p1_p2 <= 150.0:
        dk_p1_p2_new = (150.0 - dk_p1_p2)
        if p1 == 4: # Jupiter
            dk_p1_p2_new += 30
    elif dk_p1_p2 >= 150.01 and dk_p1_p2 <= 180.0:
        dk_p1_p2_new = 2.0*(dk_p1_p2 - 150)
    elif dk_p1_p2 >= 180.01 and dk_p1_p2 <= 300.0:
        dk_p1_p2_new = 0.5*(300.0 - dk_p1_p2)
        if p1 == 2 and (dk_p1_p2 > 210.01 and dk_p1_p2 < 240.01) : # Mars
            dk_p1_p2_new += 15
        if p1 == 4 and (dk_p1_p2 > 240.01 and dk_p1_p2 < 270.01) : # Mars
            dk_p1_p2_new += 30
        if p1 == 6 and (dk_p1_p2 > 270.01 and dk_p1_p2 < 300.01) : # Mars
            dk_p1_p2_new += 45
    else:
        dk_p1_p2_new = 0.0
    if p1 not in [3,4]:
        dk_p1_p2_new = round(dk_p1_p2_new*0.25,2)
    return dk_p1_p2_new
def bhava_drishti_bala(jd,place):
    """ TODO: Check if Bhava Drishi bala is same as Aspect Relationship Table??? """
    return _bhava_drik_bala(jd, place)
def _bhava_drik_bala(jd,place):
    dk = [[ 0 for _ in range(7)] for _ in range(12)]
    pp = charts.rasi_chart(jd, place)
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(pp)
    pp = pp[1:-2]
    subha_grahas = [1,3,4,5] ; asubha_grahas = [0,2,6]
    """ 
        TODO: Find out the aspect dictionary as below from the chart
        For example Sun aspects Moon and Venus, Moon aspects All but itelf...
        How to find this? Graha/Rasi Drishti?
        (1) Jupiter and Venus are natural benefics (saumya grahas or subha grahas).
        Mercury becomes a natural benefic when he is alone or with more natural
        benefics. Waxing Moon of Sukla paksha is a natural benefic.
        (2) Sun, Mars, Rahu and Ketu are natural malefics (kroora grahas or paapa grahas).
        Mercury becomes a natural malefic when he is joined by more natural malefics.
        Waning Moon of Krishna paksha is a natural malefic.        
    """ 
    grp,ghp,gpp = house.graha_drishti_from_chart(house_planet_dict)
    rrp,rhp,rpp = house.raasi_drishti_from_chart(house_planet_dict)
    planet_house_aspects = {}
    for planet in range(7):
        planet_house_aspects[planet] = sorted(list(set(ghp[planet]+rhp[planet])))
        planet_house_aspects[planet] = [int(p) for p in planet_house_aspects[planet] if p not in [const._ascendant_symbol,'7','8']]
    bm = drik.bhaava_madhya(jd, place)
    for h in range(12): # Aspected Planet
        h_mid = bm[h]
        for p in range(7): # Aspecting Planet
            if (h+1) in planet_house_aspects[p]:
                p_long = pp[p][1][0]*30+pp[p][1][1]
                dk_h_p = round((360.0+h_mid-p_long)%360,2)
                dk_h_p = __bhava_drik_bala_calc_1(dk_h_p,p)
            else:
                dk_h_p = 0.0
            dk[h][p] = round(dk_h_p,2)
    import numpy as np
    dkp = [0 for _ in range(12)] ; dkm = [0 for _ in range(12)]; dk_final = [0 for _ in range(12)]
    for row in range(12):
        for col in range(7):
            if col in subha_grahas:
                dkp[row] += dk[row][col] 
            if row in asubha_grahas:
                dkm[row] += dk[row][col]
            dk_final[row] = round((dkp[row] - dkm[row])/4,2) 
    return dk_final
def bhava_bala(jd,place):
    """
        Computes bhava bala
        Returns bhava bala as list of bhava bala followed by list of bhava bala in rupas
    """
    bab = _bhava_adhipathi_bala(jd, place)
    bdb = _bhava_dig_bala(jd, place)
    bdrb = _bhava_drik_bala(jd, place)
    bb = list(map(sum,zip(*[bab,bdb,bdrb])))
    bb = [round(b,2) for b in bb]
    bb_rupas = [round(b/60,2) for b in bb]
    bb_strength = [round(b/const.minimum_bhava_bala_rupa,2) for b in bb_rupas]
    return [bb,bb_rupas,bb_strength]
if __name__ == "__main__":
    dob = (1996,12,7)
    tob = (10,34,0)
    place = drik.Place('Chennai',13.0878,80.2785,5.5)
    jd = utils.julian_day_number(dob, tob)
    pp = charts.rasi_chart(jd, place)
    print(pp[1:10])
    pa = planet_aspect_relationship_table(pp,include_houses=True)
    print(pa)
    exit()
    ur = _uccha_rashmi(pp)
    print(ur)
    print(_cheshta_rashmi(jd,place))
    print(_subha_rashmi(jd,place))
    print(_ishta_phala(jd,place))
    exit()
    rasi_chart = utils.get_house_planet_list_from_planet_positions(pp)
    print('rasi chart',rasi_chart)
    from jhora.horoscope.transit import tajaka
    print(tajaka.planet_aspects_from_chart(rasi_chart))
    #exit()
    print('_bhava_adhipathi_bala',_bhava_adhipathi_bala(jd, place))
    print('_bhava_dig_bala',_bhava_dig_bala(jd, place))
    print('_bhava_drik_bala',_bhava_drik_bala(jd,place))
    print('bhava_bala',bhava_bala(jd, place))
    exit()
    sb = shad_bala(jd,place)
    for i,b in enumerate(['sthaana bala','kaala bala','dig bala','chesta bala','naisargika bala','drik bala','shad bala','shad bala (rupas)','shad bala (strength)']):
        print(b,sb[i])
    exit()
    """
    print('uccha bala',_uchcha_bala(pp_sv[1]))
    print('saptha vargaja bala',_sapthavargaja_bala(jd,place))
    print('ojayugama bala',_ojayugama_bala(pp_sv[1],pp_sv[9]))
    print('kendra bala',_kendra_bala(pp_sv[1]))
    print(' dreshkon bala',_dreshkon_bala(pp_sv[1]))
    """
    print('sthana bala', _sthana_bala(jd,place))
    print('dig bala',_dig_bala(jd,place))
    print('_nathonnath_bala',_nathonnath_bala(jd,place))
    print('_paksha_bala',_paksha_bala(jd,place))
    print('_tribhaga_bala',_tribhaga_bala(jd,place))
    print('_hora_bala',_hora_bala(jd,place))
    print('_abda_bala',_abda_bala(jd,place))
    print('_masa_bala',_masa_bala(jd,place))
    print('_vaara_bala',_vaara_bala(jd,place))
    print('_ayana_bala',_ayana_bala(jd,place))
    print('_yuddha_bala',_yuddha_bala(jd,place))
    print('_kaala_bala',_kaala_bala(jd,place))
    print('_cheshta_bala',_cheshta_bala(jd,place))
    print('_naisargika_bala',_naisargika_bala(jd,place))
    print('_drik_bala',_drik_bala(jd,place))


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Tara Dasa - applicable if all the four quadrants are occupied """
"""
    TODO: To implement following options for sanjay rath method:
        1. No dhasa sesham OR 2. Dasa sesham from moon as per vimsottari 
        OR 3. Dhasa sesham from moon but in reverse for apasavya nakshatras
"""

from jhora import const, utils
from jhora.panchanga import drik
from jhora.horoscope.chart import charts, house

dhasa_adhipathi_dict_sanjay_rath = {5:20,1:10,8:7,6:19,4:16,3:17,7:18,2:7,0:6}
dhasa_adhipathi_dict_parasara = {5:20,0:6,1:10,2:7,7:18,4:16,6:19,3:17,8:7}
human_life_span = sum(dhasa_adhipathi_dict_parasara.values())
year_duration = const.sidereal_year

def _next_adhipati(lord,dirn=1,dhasa_method=1):
    dhasa_adhipathi_dict = dhasa_adhipathi_dict_sanjay_rath if dhasa_method==1 else dhasa_adhipathi_dict_parasara
    """Returns next lord after `lord` in the adhipati_list"""
    current = list(dhasa_adhipathi_dict.keys()).index(lord)
    next_lord = list(dhasa_adhipathi_dict.keys())[((current + dirn) % len(dhasa_adhipathi_dict))]
    return next_lord
def _antardhasa(lord,dhasa_method=1):
    dhasa_adhipathi_dict = dhasa_adhipathi_dict_sanjay_rath if dhasa_method==1 else dhasa_adhipathi_dict_parasara
    _bhukthis = []
    for _ in range(len(dhasa_adhipathi_dict)):
        _bhukthis.append(lord)
        lord = _next_adhipati(lord,dhasa_method=dhasa_method)
    return _bhukthis
def _dhasa_start(jd,place,period,star_position_from_moon=1,divisional_chart_factor=1,chart_method=1):
    one_star = (360 / 27.)
    planet_positions = charts.divisional_chart(jd, place, divisional_chart_factor=divisional_chart_factor,
                                               chart_method=chart_method)
    moon = planet_positions[2][1][0]*30+planet_positions[2][1][1]+(star_position_from_moon-1)*one_star
    nak = int(moon / one_star); rem = (moon - nak * one_star)
    period_elapsed = rem / one_star * period # years
    period_elapsed *= year_duration        # days
    start_date = jd - period_elapsed      # so many days before current day
    return start_date
        
def get_dhasa_bhukthi(dob,tob,place,divisional_chart_factor=1,chart_method=1,years=1,months=1,sixty_hours=1,
                      include_antardasa=True,dhasa_method=1):
    """
        provides Tara dhasa bhukthi for a given date in julian day (includes birth time)
        @param dob: date of birth as tuple
        @param tob: time of birth as tuple
        @param place: Place as tuple (place name, latitude, longitude, timezone) 
        @param divisional_chart_factor Default=1 
            1=Raasi, 9=Navamsa. See const.division_chart_factors for options
        @param chart_method: Default=1, various chart methods available for each div chart. See charts module
        @param years: Yearly chart, # of years from date of birth
        @param months: Monthly chart, # of months from date of birth
        @param sixty_hours: 60 hour chart, # 60hrs from date of birth
        @param include_antardhasa True/False. Default=True
        @param dhasa_method: 
            1=>Sanjay Rath method dhasa order 5,1,8,6,4,3,7,2,0  (Default)
            2=>Parasara method dhasa order 5,0,1,2,7,4,6,3,8
        @return: a list of [dhasa_lord,bhukthi_lord,bhukthi_start, duration]
          Example: [ [7, 5, '1915-02-09',0.25], [7, 0, '1917-06-10',0.25], ...]
    """
    dhasa_adhipathi_dict = dhasa_adhipathi_dict_sanjay_rath if dhasa_method==1 else dhasa_adhipathi_dict_parasara
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE, 
                            divisional_chart_factor=divisional_chart_factor, chart_method=chart_method,
                            years=years,months=months, sixty_hours=sixty_hours)[:const._pp_count_upto_ketu] # Exclude Western Planets
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    asc_house = planet_positions[0][1][0]
    ds = sorted([h_to_p[(h+asc_house)%12].split('/') for h in [0,3,6,9] if h_to_p[(h+asc_house)%12] != ''],key=len,reverse=True)
    ds = utils.flatten_list(ds)
    """ TODO If ds is empty list - what to do? """
    if len(ds)==0:
        print('tara dhasa ds list is empty, returning empty list')
        return []
    if 'L' in ds:
        ds.remove('L')
    if len(ds)==0:
        print('tara dhasa ds list is empty, returning empty list')
        return []
    if len(ds) >= 2:
        ds1 = ds; sp = int(ds1[0])
        for p in range(1,len(ds1)):
            sp = house.stronger_planet_from_planet_positions(planet_positions, int(ds1[p]), int(sp))
        dhasa_lord = sp
    else:#if len(ds)==1:
        dhasa_lord = int(ds[0])#int(ds[0][0])
    _dhasa_duration = dhasa_adhipathi_dict[dhasa_lord]
    start_jd = _dhasa_start(jd_at_dob,place, _dhasa_duration, star_position_from_moon=1,
                            divisional_chart_factor=divisional_chart_factor,chart_method=chart_method)
    dhasa_info = []
    for _ in range(len(dhasa_adhipathi_dict)):
        _dhasa_lord_duration = dhasa_adhipathi_dict[dhasa_lord]
        bhukthis = _antardhasa(dhasa_lord,dhasa_method=dhasa_method)
        if include_antardasa:
            for bhukthi_lord in bhukthis:
                _bhukthi_duration = dhasa_adhipathi_dict[bhukthi_lord]
                factor = _bhukthi_duration *  _dhasa_lord_duration / human_life_span
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,_dhasa_duration))
                start_jd += factor * year_duration
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((dhasa_lord,dhasa_start,_dhasa_duration))
            lord_duration = dhasa_adhipathi_dict[dhasa_lord]
            start_jd += lord_duration * year_duration
        dhasa_lord = _next_adhipati(dhasa_lord,dhasa_method=dhasa_method)
        _dhasa_duration = dhasa_adhipathi_dict[dhasa_lord]
    return dhasa_info
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests._STOP_IF_ANY_TEST_FAILED = False
    pvr_tests.tara_dhasa_test()


#  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  #
#  PyJHora - Based on the book Vedic Astrology - An Integrated Approach, PVR Narasimha Rao        #
#  Copyright © 2022 Sundar Sundaresan <Sundaram.Sundaresan@gmail.com>.                           #
#                                                                                                #
#  This program is free software: you can redistribute it and/or modify it under the terms of    #
#  the GNU General Public License as published by the Free Software Foundation, either version   #
#  3 of the License, or (at your option) any later version.                                      #
#                                                                                                #
#  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;     #
#  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.     #
#  See the GNU General Public License for more details.                                          #
#  																								 #
#  Vedic Astrology was givem to us by our ancient sages without any commerical benefits.         #
#  Let us also do the same without any commercial benfits and distribute this free. THANK YOU    #
#  You should have received a copy of the GNU General Public License along with this program.    #
#  If not, see <http://www.gnu.org/licenses/>.                                                   #
#  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  #

name = "PyJHora"

version = "4.0.3"

author = "Sundar Sundaresan"

author_email = "Sundaram.Sundaresan@gmail.com"

description = "A Python package for generating Indian style calendar, panchang, horoscope and compatibility"

url = "https://github.com/naturalstupid/PyJHora"

project_urls = {
    "Source Code": "https://github.com/naturalstupid/PyJHora",
    "Documentation": "https://github.com/naturalstupid/PyJHora",
}

#install_requires = ['itertools', "configparser", 'operator', 'collections', 'enum', 'csv', 'numpy', 'scamp', 'math', 're', 'regex', 'random',]
install_requires = ['swisseph', 'geopy','pytz','PyQt6' ]

#extras_require = { }

package_data = {
    'horoscope': ["data/*",'lang/*','images/*']
}

classifiers = [
    "Programming Language :: Python :: 3.6",
    "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    "Operating System :: OS Independent",
]



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from PyQt6.QtWidgets import QLineEdit, QApplication, QLabel, QHBoxLayout, QVBoxLayout, QPushButton,\
                            QComboBox, QDialog
from PyQt6.QtCore import Qt
from jhora import utils
from jhora.panchanga import drik
import datetime
_main_window_width = 550
_main_window_height = 150
_CALCULATE_WAIT_MSG = '<b>Calculating... Results will be updated here. Please wait...</b>'
class ConjunctionDialog(QDialog):
    """
        Planetary Conjunction Dialog
    """
    def __init__(self,current_date_jd,place:drik.Place,planet1=6, chart_title='Planetary Conjunctions',
                 entry_type=0,planet2=4,raasi=None):
        """
            Entry Type: 0=> Planetary Conjunctions; 1=> Planet Entry/Transit 2=> Planet Vakra Gathi Change
        """
        super().__init__()
        self._current_date_jd = current_date_jd
        self._conjunction_date_jd = current_date_jd
        self.Place = place
        self._planet1 = planet1
        self._chart_title = chart_title
        self._entry_type = entry_type
        self._accept_clicked = False
        self._separation_angle_index = 0
        if entry_type==0:
            self._planet2 = planet2
        elif entry_type==1:
            self._raasi = raasi
        self.res = utils.resource_strings
        self._create_ui()
    def _create_ui(self):
        v_layout = QVBoxLayout()
        self._current_date_label = QLabel(self.res['current_date_str'])
        y,m,d,fh = utils.jd_to_gregorian(self._current_date_jd)
        hh,mm,ss = utils.to_dms(fh,as_string=False)
        h_layout = QHBoxLayout()
        h_layout.addWidget(self._current_date_label)
        self._year_text = QLineEdit(str(y));h_layout.addWidget(self._year_text)
        self._month_text = QLineEdit(str(m)); h_layout.addWidget(self._month_text)
        self._day_text = QLineEdit(str(d)); h_layout.addWidget(self._day_text)
        self._after_before_combo = QComboBox(); h_layout.addWidget(self._after_before_combo)
        self._after_before_combo.addItems([self.res['after_str'],self.res['before_str']])
        v_layout.addLayout(h_layout)
        h_layout = QHBoxLayout()
        if self._entry_type==0:
            self._sep_angle_combo = QComboBox()
            self._sep_angle_list = self.res['planet_separation_angle_list'].split(',')
            self._sep_angle_combo.addItems(self._sep_angle_list)
            self._sep_angle_combo.setCurrentIndex(0)
            self._separation_angle = self._sep_angle_combo.currentIndex()*30.
            h_layout.addWidget(self._sep_angle_combo)
        self._planet1_combo = QComboBox()
        if self._entry_type==2:
            self._planet1_list = utils.PLANET_NAMES[2:7]
        else:
            self._planet1_list = [self.res['ascendant_str']]+utils.PLANET_NAMES[:9]
        self._planet1_combo.addItems(self._planet1_list)
        self._planet1_combo.setCurrentIndex(0)
        if self._planet1=='L' or utils.PLANET_NAMES[self._planet1] in self._planet1_list:
            self._planet1_combo.setCurrentText(utils.PLANET_NAMES[self._planet1])
        h_layout.addWidget(self._planet1_combo)
        if self._entry_type==0:
            self._planet2_combo = QComboBox()
            self._planet1_combo.currentIndexChanged.connect(self._update_planet2_list)
            self._update_planet2_list()
            h_layout.addWidget(self._planet2_combo)
        elif self._entry_type==1:
            self._raasi_combo = QComboBox()
            self._raasi_combo.addItems([self.res['raasi_str']]+utils.RAASI_LIST)
            h_layout.addWidget(self._raasi_combo)
        v_layout.addLayout(h_layout)
        self._results_text = QLabel(self._after_before_combo.currentText()+'occurs at:')
        self._results_text.setEnabled(False)
        v_layout.addWidget(self._results_text)
        h_layout = QHBoxLayout()
        self._find_button = QPushButton(self.res['compute_str']); h_layout.addWidget(self._find_button)
        self._find_button.clicked.connect(self._calculate_clicked)
        self._accept_button = QPushButton(self.res['accept_str']); h_layout.addWidget(self._accept_button)
        self._accept_button.clicked.connect(self._accept_and_close)
        self._close_button = QPushButton(self.res['cancel_str']); h_layout.addWidget(self._close_button)
        self._close_button.clicked.connect(self._close_dialog)
        v_layout.addLayout(h_layout)
        self.setLayout(v_layout)
        self.setFixedSize(_main_window_width,_main_window_height)
        self.setWindowTitle(self._chart_title)
        self._results_text.setText('')
        self._accept_button.setEnabled(False)
        #self._calculate_clicked()
    def closeEvent(self, *args, **kwargs):
        self._conjunction_date_jd = self._current_date_jd
        QApplication.restoreOverrideCursor()
        return QDialog.closeEvent(self, *args, **kwargs)
    def _update_planet2_list(self):
        self._planet2_combo.clear()
        _planet2_list = [self.res['ascendant_str']]+ utils.PLANET_NAMES[:9]
        _planet2_list.remove(self._planet1_combo.currentText())
        if self._planet1_combo.currentIndex()==8 or self._planet1_combo.currentIndex()==9:
            _planet2_list = _planet2_list[:7]
        self._planet2_combo.addItems(_planet2_list)
        self._planet2_combo.setCurrentIndex(0)
        if utils.PLANET_NAMES[self._planet2] in _planet2_list:
            self._planet2_combo.setCurrentText(utils.PLANET_NAMES[self._planet2])
    def _calculate_clicked(self):
        self._accept_button.setEnabled(False)
        if self._entry_type==0:
            self.setWindowTitle(self.res['planetary_conjunctions_str'])
            self._find_conjunction_date()
        elif self._entry_type==1:
            self.setWindowTitle(self.res['planet_transit_str'])
            self._find_transit_date()
        else:
            self.setWindowTitle(self.res['vakra_gathi_change_str'])
            self._find_retrogression_change_date()
        self._accept_button.setEnabled(True)
    def _find_retrogression_change_date(self):
        from jhora import const
        self._results_text.setText(_CALCULATE_WAIT_MSG)
        QApplication.processEvents()
        QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)
        self._planet1 = utils.PLANET_NAMES.index(self._planet1_combo.currentText())
        panchanga_place = self.Place
        year = int(self._year_text.text()); month = int(self._month_text.text()); day=int(self._day_text.text())
        direction = 1 if self._after_before_combo.currentIndex()==0 else -1
        panchanga_start_date = drik.Date(year,month,day+1*direction)
        cur_jd,ret_sign = drik.next_planet_retrograde_change_date(self._planet1,panchanga_start_date,panchanga_place,direction=direction)
        retStr=''  if ret_sign == 1 else const._retrogade_symbol
        self._conjunction_date_jd = cur_jd
        results = self._planet1_combo.currentText()+' '
        #print(results)
        y,m,d,fh= utils.jd_to_gregorian(cur_jd)
        results += "{0:4d}-{1:2d}-{2:2d}".format(y,m,d)+' '+utils.to_dms(fh,as_string=True)
        results += ' '+utils.PLANET_NAMES[self._planet1]+retStr
        #print(results)
        self._results_text.setText(results)
        # update the year/month/day edit boxes
        self._year_text.setText(str(y)); self._month_text.setText(str(m)); self._day_text.setText(str(d))
        QApplication.restoreOverrideCursor()        
        
    def _find_conjunction_date(self):
        #from datetime import datetime
        self._separation_angle = self._sep_angle_combo.currentIndex()*30.
        #start_time = datetime.now()
        self._results_text.setText(_CALCULATE_WAIT_MSG)
        QApplication.processEvents()
        QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)
        self._planet1 = 'L' if self._planet1_combo.currentIndex()==0 else utils.PLANET_NAMES.index(self._planet1_combo.currentText())
        self._planet2 = 'L' if self._planet2_combo.currentIndex()==0 else utils.PLANET_NAMES.index(self._planet2_combo.currentText())
        panchanga_place = self.Place
        year = int(self._year_text.text()); month = int(self._month_text.text()); day=int(self._day_text.text())  
        self._current_date_jd = utils.julian_day_number(drik.Date(year,month,day),(0,0,1/(24/60/60)))
        direction = 1 if self._after_before_combo.currentIndex()==0 else -1
        ret = drik.next_conjunction_of_planet_pair(self._current_date_jd,panchanga_place, self._planet1, self._planet2, direction=direction,separation_angle=self._separation_angle)
        self._separation_angle_index = self._sep_angle_combo.currentIndex()
        if ret==None: #Error text fixec in V3.8.1
            self._results_text.setText('Could not find planetary conjunctions for sep angle '+str(self._separation_angle)+'  Try increasing search range')
            QApplication.restoreOverrideCursor()
            return
        else:
            cur_jd,p1_long,p2_long = ret
        #end_time = datetime.now()
        #print("Elapsed", (end_time - start_time).total_seconds(),'seconds')
        self._conjunction_date_jd = cur_jd
        results = self._planet1_combo.currentText()+'/'+self._planet2_combo.currentText()+' '
        y,m,d,fh= utils.jd_to_gregorian(cur_jd)
        results += "{0:4d}-{1:2d}-{2:2d}".format(y,m,d)+' '+utils.to_dms(fh,as_string=True)
        pstr1 = self.res['ascendant_str'] if self._planet1 == 'L' else utils.PLANET_NAMES[self._planet1]
        pstr2 = self.res['ascendant_str'] if self._planet2 == 'L' else utils.PLANET_NAMES[self._planet2]
        results += ' '+self.res['longitude_str']+' '+\
                    pstr1+':'+utils.to_dms(p1_long,is_lat_long='plong')+' ' + \
                    pstr2+':'+utils.to_dms(p2_long,is_lat_long='plong')
        self._results_text.setText(results)
        # update the year/month/day edit boxes
        self._year_text.setText(str(y)); self._month_text.setText(str(m)); self._day_text.setText(str(d))
        QApplication.restoreOverrideCursor()
        return
    def _find_transit_date(self):
        from datetime import datetime
        start_time = datetime.now()
        self._results_text.setText(_CALCULATE_WAIT_MSG)
        QApplication.processEvents()
        QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)
        self._planet1 = 'L' if self._planet1_combo.currentIndex()==0 else utils.PLANET_NAMES.index(self._planet1_combo.currentText())
        panchanga_place = self.Place
        year = int(self._year_text.text()); month = int(self._month_text.text()); day=int(self._day_text.text())  
        direction = 1 if self._after_before_combo.currentIndex()==0 else -1
        panchanga_start_date = drik.Date(year,month,day)
        jd = utils.julian_day_number(panchanga_start_date,(0,0,0))+1/(24/60/60)
        self._raasi = None if self._raasi_combo.currentIndex()==0 else self._raasi_combo.currentIndex()
        cur_jd,p1_long = drik.next_planet_entry_date(self._planet1,jd,panchanga_place,
                                                     direction=direction,raasi=self._raasi)
        end_time = datetime.now()
        print("Elapsed", (end_time - start_time).total_seconds(),'seconds')
        self._conjunction_date_jd = cur_jd
        results = self._planet1_combo.currentText()+' '
        #print(results)
        y,m,d,fh= utils.jd_to_gregorian(cur_jd)
        p1_rasi,p1_long = drik.dasavarga_from_long(p1_long, divisional_chart_factor=1)
        results += "{0:4d}-{1:2d}-{2:2d}".format(y,m,d)+' '+utils.to_dms(fh,as_string=True)
        results += ' '+self.res['longitude_str']+' :'+ utils.RAASI_LIST[p1_rasi]+' ' + utils.to_dms(p1_long,is_lat_long='plong')+' '
        #print(results)
        self._results_text.setText(results)
        # update the year/month/day edit boxes
        self._year_text.setText(str(y)); self._month_text.setText(str(m)); self._day_text.setText(str(d))
        QApplication.restoreOverrideCursor()        
    def _accept_and_close(self):
        self._accept_clicked = True
        #y,m,d,fh = utils.jd_to_gregorian(self._conjunction_date_jd)
        #print(y,m,d,utils.to_dms(fh))
        self.accept()
    def _close_dialog(self):
        self._accept_clicked = False
        self.reject()
def show(current_date_jd,place:drik.Place,planet1=6,planet2=4, chart_title='Planetary Conjunctions',entry_type=0):
    import sys
    def except_hook(cls, exception, traceback):
        sys.__excepthook__(cls, exception, traceback)
    sys.excepthook = except_hook
    dlg = ConjunctionDialog(current_date_jd,place,planet1,chart_title,entry_type,planet2)
    ret = dlg.exec()
    print('exec return value',ret)
    return ret
if __name__ == "__main__":
    import sys
    def except_hook(cls, exception, traceback):
        print('exception called')
        sys.__excepthook__(cls, exception, traceback)
    sys.excepthook = except_hook
    lang = 'ta'; utils.set_language(lang)
    App = QApplication(sys.argv)
    jd = utils.julian_day_number((1996,12,7), (10,34,0))
    place = drik.Place('Chennai,India',13.0878,80.2785,5.5)
    _chart_title = 'Planet entry'
    entry_type = 2
    show(jd, place,entry_type=entry_type,chart_title=_chart_title)
    exit()
    chart = ConjunctionDialog(jd,place,entry_type=entry_type)
    chart.show()
    sys.exit(App.exec())




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import json
from jhora.horoscope.chart import charts, house
from jhora import utils, const
from jhora.panchanga import drik
_lang_path = const._LANGUAGE_PATH

def get_dosha_resources(language='en'):
    """
        get yoga names from yoga_msgs_<lang>.txt
        @param language: Two letter language code. en, hi, ka, ta, te
        @return json strings from the resource file as dictionary 
    """
    json_file = _lang_path + const._DEFAULT_DOSHA_JSON_FILE_PREFIX+language+'.json'
    f = open(json_file,"r",encoding="utf-8")
    msgs = json.load(f)
    return msgs
def kala_sarpa(house_to_planet_list):
    """ Returns kala Sarpa Dosha True or False 
        If True type kala sarpa dosha can be obtained from the Rahu's house number (1..12)
        as the index of the array from dosha_msgs_<lang> file
    """
    p_to_h = utils.get_planet_to_house_dict_from_chart(house_to_planet_list)
    rahu_house = p_to_h[7]
    kpdc1 = all([any([p_to_h[ph]==(rahu_house+rkh)%12 for rkh in [*range(7)]]) for ph in [*range(7)]])
    ketu_house = p_to_h[8]
    kpdc2 = all([any([p_to_h[ph]==(ketu_house+rkh)%12 for rkh in [*range(7)]]) for ph in [*range(7)]])
    #print('rahu_house',rahu_house,'ketu_house',ketu_house)
    return kpdc1 or kpdc2
def manglik(planet_positions,manglik_reference_planet='L',include_lagna_house=False,
            include_2nd_house=True,apply_exceptions=True):
    """ Sanjay Rath (https://srath.com/jyoti%E1%B9%A3a/amateur/ma%E1%B9%85galika-do%E1%B9%A3a/)
        says lagna house not in ancient text. So default include_lagna_house set to False
        Similarly some astrolgers use reference planet as manglik_reference_planet=1 (moon) or 
        manglik_reference_planet=5 (venus) instead of lagna
        include_2nd_house => South India considers also 2nd house (Default = True)
        apply_exceptions => BV Raman has suggested exemptions for manglik rule. (Default - True)
        @return: [Manglik=True/False,Exceptions-True/False,[Exception Indices or -1]]
                Exception index = 0 => No Exceptions
                Exception index >1 => Exception indices can be mapped to dosha_msgs_<lang>.json file strings.
    """
    house_to_planet_list = utils.get_house_planet_list_from_planet_positions(planet_positions)
    #house_to_planet_list = ['L','7','0/1','5/6','2','3','4','8','','','','']
    p_to_h = utils.get_planet_to_house_dict_from_chart(house_to_planet_list)
    manglik_houses = [4,7,8,12]
    if include_2nd_house:
        manglik_houses = [2]+manglik_houses
    if include_lagna_house:
        manglik_houses = [1]+manglik_houses
    from_house = p_to_h[manglik_reference_planet] ; mars_house = p_to_h[2]
    mars_house_from_ref = house.get_relative_house_of_planet(from_house,mars_house)
    _manglik = mars_house_from_ref in manglik_houses
    if apply_exceptions and _manglik:
        _me = _manglik_exceptions(planet_positions)
        return [_manglik,_me[0],_me[1]]
    else:
        return [_manglik,False,[]]
def _manglik_exceptions(planet_positions):
    """
        BV Raman Exceptions:
        1. Mars in Leo/Simmam or Aquarious (Kumbam)
        2. Mars in 2nd house and in signs of Gemini/Mithuna or Virgo/Kanni
        3. Mars in 4th house and in signs of Aries/Mesham or Scorpio/Vrichigam
        4. Mars in 7th house and in signs of Cancer/Kataka or Capricorn/Makaram
        5. Mars in 8th house and in signs of Sagitarius/Dhanusu or Pisces/Meenam
        6. Mars in 12th house and in signs of Taurus/Rishabam or Libra/Thulam
        7. Mars is in association or aspected by Jupiter or Saturn
        8. Retrograde Mars
        9. Mars is weak (combust, Rasi Sandhi etc)
        10. Mars is lagna lord.
        11. Dispositor of Mars is neecha or associated with strong benefic
        12. Mars in own house, exalted or in friend' house - reduced effects
        13. Mars in movable - reduced effects
        14. Dispositor of Mars is in Quad or Trine
        15. If Lagnam is in Cancer/Kataka or Leo, then Mars is yoga karaka causes no dosha.
        16. Since Mars cojoins with Jupiter or moon, it reduces the dosha
        17. When Jupiter or Venus is in Lagna it reduces dosha
    """
    house_to_planet_list = utils.get_house_planet_list_from_planet_positions(planet_positions)
    p_to_h = utils.get_planet_to_house_dict_from_chart(house_to_planet_list)
    mars_house = p_to_h[2] ; lagna_house = p_to_h['L']; mars_long = planet_positions[3][1][1]
    mars_from_lagna = house.get_relative_house_of_planet(lagna_house,mars_house)
    _me = [] ; _me_details = []
    c1 = mars_house in [4,10] ; _me.append(c1)
    c2 = mars_from_lagna ==2 and mars_house in [2,5] ; _me.append(c2)
    c3 = mars_from_lagna ==4 and mars_house in [0,7] ; _me.append(c3)
    c4 = mars_from_lagna ==7 and mars_house in [3,9] ; _me.append(c4)
    c5 = mars_from_lagna ==8 and mars_house in [8,11] ; _me.append(c5)
    c6 = mars_from_lagna ==12 and mars_house in [1,6] ; _me.append(c6)
    c7 = len(house.associations_of_the_planet(planet_positions, 2))>0 ; _me.append(c7)
    c8 = 2 in charts.planets_in_retrograde(planet_positions) ; _me.append(c8)
    c9_1 = 2 in charts.planets_in_combustion(planet_positions)
    c9_2 = mars_long < const.rasi_sandhi_duration or mars_long > (30.0-const.rasi_sandhi_duration)
    c9 = c9_1 or c9_2 ; _me.append(c9)
    c10 = house.house_owner_from_planet_positions(planet_positions, lagna_house)==2; _me.append(c10)
    c11 = False ; _me.append(c11)
    c12 = const.house_strengths_of_planets[2][mars_house] >= const._FRIEND; _me.append(c12)
    c13 = mars_house in const.movable_signs; _me.append(c13)
    c14 = False ; _me.append(c14)
    c15 = lagna_house in [3,4] ; _me.append(c15)
    c16 = mars_house in [p_to_h[4], p_to_h[5]]; _me.append(c16)
    c17 = lagna_house in [p_to_h[4], p_to_h[5]]; _me.append(c17)
    _me_i = []
    have_exceptions = any(_me)
    if have_exceptions:
        _me_i = [i+1 for i,m in enumerate(_me) if m]
    return [have_exceptions,_me_i]
def pitru_dosha(planet_positions):
    """
        returns True/False if pitru/pitra dosha
        From timesofindia:
            Sun, moon or Rahu in 9th house
            Ketu in 4th house
            sun, moon, rahu or ketu afflicted by malefic planets like Mars or Saturn
            Venus, Mercury, Rahu or any of these two in 2nd, 5th, 9th or 12th    
        Sun or Moon is in conjunction with Rahu or Ketu
    """
    house_planet_list = utils.get_house_planet_list_from_planet_positions(planet_positions)
    planet_house_dict = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    ninth_house = (planet_house_dict['L']+8)%12
    pd = []
    pd1 = planet_house_dict[0] == ninth_house or planet_house_dict[1] == ninth_house or planet_house_dict[7] == ninth_house
    pd.append(pd1)
    pd2 = planet_house_dict[7] == (planet_house_dict['L']+3)%12
    pd.append(pd2)
    pd3 = any([2 in house.associations_of_the_planet(planet_positions, p1) or 6 in house.associations_of_the_planet(planet_positions, p1) for p1 in [0,1,7,8]])
    pd.append(pd3)
    pd4 = any([sum([planet_house_dict[p1]==(planet_house_dict['L']+h-1)%12 for p1 in [3,5,7]])>1 for h in [2,5,9,12] ])
    pd.append(pd4)
    pd5 = any([any([planet_house_dict[p1]==planet_house_dict[p2] for p2 in [7,8]]) for p1 in [0,1] ])
    pd.append(pd5)
    pdc = any(pd)
    if pdc:
        return [pdc, [i+1 for i,m in enumerate(pd) if m]]
    else:
        return [False,[]]
def guru_chandala_dosha(planet_positions):
    """ returns True/False if guru chandal dosha presents in the chart
        if Rahu/Keti conjoins Jupiter - this dosha exists
        @return True/False if chandala dosha, True/False if Jupiter stronger, Rahu/Ketu whicever conjoins Jupiter
    """
    rahu_house = planet_positions[8][1][0] ; ketu_house = planet_positions[9][1][0]
    jupiter_house = planet_positions[5][1][0]
    jupiter_is_strong = False
    if jupiter_house == rahu_house :
        jupiter_is_strong = house.stronger_planet_from_planet_positions(planet_positions, 4, 7)==4
        return True, jupiter_is_strong
    elif jupiter_house == ketu_house :
        jupiter_is_strong = house.stronger_planet_from_planet_positions(planet_positions, 4, 8)==4
        return True, jupiter_is_strong
    else:
        return False,False
def kalathra(planet_positions,reference_planet='L'):
    """
        The placement of malefic planets Mars, Saturn, Sun, Rahu, and Ketu in the 
        1st, 2nd, 4th, 7th, 8th, or 12th house from the Ascendant (Lagna) signifies Kalathra Dosha.
        @param planet_positions:
        @param reference planet - default = 'L' for lagnam. For chandran/moon use the value 1 :  
        @return: True/False if kalathra dosha exists of not
    """
    reference_house = (planet_positions[0][1][0]+6)%12
    if reference_planet==1:
        reference_house = (planet_positions[2][1][0]+6)%12
    kc = all([any([planet_positions[p+1][1][0]==(reference_house+h-1)%12 for h in [1,2,4,7,8,12] ]) for p in const.natural_malefics ])
    #print(kc)
    return kc
def _get_kalathra_results(planet_positions,dosha_msgs,key_str, reference_planet='L'):
    ks_results = {}
    """ get kalathra dosha """
    next_line = "<br><br>"
    ks = 'kalathra'
    ks_key = key_str
    ks_msgs = dosha_msgs[ks]
    kpd = kalathra(planet_positions, reference_planet=reference_planet)
    ks_results[ks_key] = "<html>"+ks_msgs[0]+next_line
    if kpd:
        ks_results[ks_key] = "<html>"+ks_msgs[-1]+next_line
    ks_results[ks_key] += "</html>"
    return ks_results
def ganda_moola(moon_star):
    return moon_star in const.ganda_moola_stars
def _get_ganda_moola_results(jd_at_dob,place,dosha_msgs,key_str):
    m_results = {}
    next_line = "<br><br>"
    m = "ganda_moola"
    m_key = key_str
    m_msgs = dosha_msgs[m]
    moon_star = drik.nakshatra(jd_at_dob, place)
    _gm = ganda_moola(moon_star)
    #print('_gc',_gc)
    m_results[m_key] = "<html>"+m_msgs[0]+next_line
    if _gm:
        m_results[m_key] = m_msgs[-1]+next_line
        m_results[m_key] += m_msgs[const.ganda_moola_stars.index(moon_star)+1]
    return m_results 
def _get_guru_chandala_results(planet_positions,dosha_msgs,key_str):
    m_results = {}
    next_line = "<br><br>"
    m = "guru_chandal"
    m_key = key_str
    m_msgs = dosha_msgs[m]
    rahu_house = planet_positions[8][1][0] ; ketu_house = planet_positions[9][1][0]
    jupiter_house = planet_positions[5][1][0]
    _gc = guru_chandala_dosha(planet_positions)
    #print('_gc',_gc)
    m_results[m_key] = "<html>"+m_msgs[0]+next_line
    if _gc[0]:
        m_results[m_key] = "<html>"+m_msgs[-1]+next_line
        if _gc[1]:
            if jupiter_house==rahu_house:
                m_results[m_key] += utils.resource_strings['guru_stronger_than_rahu']+next_line
            elif jupiter_house==ketu_house:
                m_results[m_key] += utils.resource_strings['guru_stronger_than_ketu']+next_line
            m_results[m_key] += m_msgs[planet_positions[5][1][0]+1] + next_line
    m_results[m_key] += "</html>"
    return m_results
def _get_kala_sarpa_results(planet_positions,dosha_msgs,key_str):
    house_to_planet_list = utils.get_house_planet_list_from_planet_positions(planet_positions)
    ks_results = {}
    """ get kala sarpa dosha """
    next_line = "<br><br>"
    ks = 'kala_sarpa'
    ks_key = key_str
    ks_msgs = dosha_msgs[ks]
    kpd = kala_sarpa(house_to_planet_list)
    ks_results[ks_key] = "<html>"+ks_msgs[0]+next_line
    if kpd:
        ks_results[ks_key] = "<html>"+ks_msgs[-1]+next_line
        rahu_house = house.get_relative_house_of_planet(planet_positions[0][1][0],planet_positions[8][1][0])
        #print('rahu house',rahu_house,ks_msgs[rahu_house])
        ks_results[ks_key] += ks_msgs[rahu_house]+next_line
    ks_results[ks_key] += "</html>"
    return ks_results
def ghata(planet_positions):
    """
        Mars/Saturn conjunction results in ghata dosha
        @return: True/False
    """
    return planet_positions[3][1][0]==planet_positions[7][1][0]
def shrapit(planet_positions):
    """
        Rahu/Saturn conjunction results in Shrapit dosha
        @return: True/False
    """
    return planet_positions[8][1][0]==planet_positions[7][1][0]
def _get_ghata_results(planet_positions,dosha_msgs,key_str):
    house_to_planet_list = utils.get_house_planet_list_from_planet_positions(planet_positions)
    ks_results = {}
    """ get ghata dosha """
    next_line = "<br><br>"
    ks = 'ghata'
    ks_key = key_str
    ks_msgs = dosha_msgs[ks]
    kpd = ghata(planet_positions)
    ks_results[ks_key] = "<html>"+ks_msgs[0]+next_line
    if kpd:
        ks_results[ks_key] = "<html>"+ks_msgs[-1]+next_line
        mars_house = house.get_relative_house_of_planet(planet_positions[0][1][0], planet_positions[3][1][0])
        ks_results[ks_key] += ks_msgs[mars_house]+next_line
    ks_results[ks_key] += "</html>"
    return ks_results
def _get_shrapit_results(planet_positions,dosha_msgs,key_str):
    house_to_planet_list = utils.get_house_planet_list_from_planet_positions(planet_positions)
    ks_results = {}
    """ get shrapt dosha """
    next_line = "<br><br>"
    ks = 'shrapit'
    ks_key = key_str
    ks_msgs = dosha_msgs[ks]
    kpd = shrapit(planet_positions)
    ks_results[ks_key] = "<html>"+ks_msgs[0]+next_line
    if kpd:
        ks_results[ks_key] = "<html>"+ks_msgs[-1]+next_line
        saturn_house = house.get_relative_house_of_planet(planet_positions[0][1][0], planet_positions[7][1][0])
        ks_results[ks_key] += ks_msgs[saturn_house]+next_line
    ks_results[ks_key] += "</html>"
    return ks_results
def _get_manglik_results(planet_positions, dosha_msgs,key_str):
    m_results = {}
    next_line = "<br><br>"
    m = "manglik"
    m_key = key_str
    m_msgs = dosha_msgs[m]
    _manglik = manglik(planet_positions)
    m_results[m_key] = "<html>"+m_msgs[0]+next_line
    if _manglik[0]:
        m_results[m_key] = "<html>"+m_msgs[-1]+next_line
        mars_house = house.get_relative_house_of_planet(planet_positions[0][1][0],planet_positions[3][1][0])
        #print('mars_house',mars_house)
        m_results[m_key] += m_msgs[mars_house]+next_line
        """ Check exceptions """
        e_msgs = dosha_msgs['manglik_exceptions']
        exp = e_msgs[0]
        if _manglik[1]:
            exp = e_msgs[-1]+next_line
            for _me in _manglik[2]:
                exp += "\t"+e_msgs[_me]+next_line
        m_results[m_key] += exp
    m_results[m_key] += "</html>"
    return m_results
def _get_pitru_results(planet_positions,dosha_msgs,key_str):
    house_to_planet_list = utils.get_house_planet_list_from_planet_positions(planet_positions)
    ks_results = {}
    next_line = "<br><br>"
    ks = 'pitru'
    ks_key = key_str
    ks_msgs = dosha_msgs[ks]
    kpd = pitru_dosha(planet_positions)
    ks_results[ks_key] = "<html>"+ks_msgs[0]+next_line
    if kpd[0]:
        ks_results[ks_key] = "<html>"+ks_msgs[-1]+next_line
        for m in kpd[1]:
            ks_results[ks_key] += ks_msgs[m]+next_line
    ks_results[ks_key] += "</html>"
    return ks_results
def get_dosha_details(jd_at_dob,place_as_tuple,language=const._DEFAULT_LANGUAGE):
    dosha_msgs = get_dosha_resources(language)
    #print(dosha_msgs)
    planet_positions = charts.rasi_chart(jd_at_dob, place_as_tuple)
    house_to_planet_list = utils.get_house_planet_list_from_planet_positions(planet_positions)
    dosha_results = {}
    """ get kala sarpa dosha """
    key_str = utils.resource_strings['kala_sarpa_dosha_str']
    ks_results = _get_kala_sarpa_results(planet_positions, dosha_msgs,key_str)
    dosha_results.update(ks_results)
    """ get manglik """
    key_str = utils.resource_strings['manglik_dosha_str']
    m_results = _get_manglik_results(planet_positions, dosha_msgs,key_str)
    dosha_results.update(m_results)
    """ get pitru """
    key_str = utils.resource_strings['pitru_dosha_str']
    m_results = _get_pitru_results(planet_positions,dosha_msgs,key_str)
    dosha_results.update(m_results)
    """ get guru chandala """
    key_str = utils.resource_strings['guru_chandala_dosha_str']
    m_results = _get_guru_chandala_results(planet_positions,dosha_msgs,key_str)
    dosha_results.update(m_results)
    """ get ganda moola """
    key_str = utils.resource_strings['ganda_moola_dosha_str']
    m_results = _get_ganda_moola_results(jd_at_dob,place_as_tuple,dosha_msgs,key_str)
    dosha_results.update(m_results)
    """ get kalathra """
    key_str = utils.resource_strings['kalathra_dosha_str']
    m_results = _get_kalathra_results(planet_positions, dosha_msgs, key_str, reference_planet='L')
    dosha_results.update(m_results)
    """ get ghata """
    key_str = utils.resource_strings['ghata_dosha_str']
    m_results = _get_ghata_results(planet_positions, dosha_msgs, key_str)
    dosha_results.update(m_results)
    """ get shrapit """
    key_str = utils.resource_strings['shrapit_dosha_str']
    m_results = _get_shrapit_results(planet_positions, dosha_msgs, key_str)
    dosha_results.update(m_results)
    return dosha_results
if __name__ == "__main__":
    lang = 'ta'
    utils.set_language(lang)
    res = utils.resource_strings
    from jhora.horoscope.chart import charts
    from jhora.panchanga import drik
    dob = (1996,12,7); tob = (10,34,0); jd_at_dob = utils.julian_day_number(dob, tob)
    place_as_tuple = drik.Place('Chennai, India',13.0878,80.2785,5.5)
    pp = charts.rasi_chart(jd_at_dob, place_as_tuple)
    h_to_p = utils.get_house_planet_list_from_planet_positions(pp)
    print(h_to_p)
    tob_hrs = 10+34/60.0
    sunrise_hrs = drik.sunrise(jd_at_dob, place_as_tuple)[0]
    print(tob_hrs,sunrise_hrs,utils.udhayadhi_nazhikai(tob, sunrise_hrs))
    exit()
    #pp = [['L',(1,0)],[0,(2,0)],[1,(3,0)],[2,(3,0)],[3,(4,0)],[4,(5,2.0)],[5,(6,0)],[6,(7,0)],[7,(5,0)],[8,(0,0)]]
    #dosha_msgs = get_dosha_resources(lang)
    #key_str = utils.resource_strings['guru_chandala_dosha_str']
    ks = get_dosha_details(jd_at_dob, place_as_tuple, language=lang)
    print(ks)
    exit()
    #pp = [['L',(9,0.0)],[0,(7,0.0)],[1,(6,0.0)],[2,(4,0.0)],[3,(8,0.0)],[4,(8,0.0)],[5,(6,0.0)],[6,(11,0.0)],[7,(5,0.0)],[8,(11,0.0)]]
    h_to_p = ['','','','','2','7','1/5','0','3/4','L','','6/8']
    #pp = [['L',(0,0.0)],[0,(9,0.0)],[1,(9,0.0)],[2,(0,0.0)],[3,(10,0.0)],[4,(11,0.0)],[5,(1,0.0)],[6,(10,0.0)],[7,(8,0.0)],[8,(2,0.0)]]
    #h_to_p = ['L/2','5','8','','','','','','7','0/1','6/3','4']
    print('kala_sarpa',kala_sarpa(h_to_p))
    mng = manglik(pp,manglik_reference_planet='L')
    print('manglik dosha',mng)
    exit()
    #pp = [['L',(1,0.0)],[0,(1,0.0)],[0,(7,0.0)],[2,(6,0.0)],[3,(5,0.0)],[4,(4,0.0)],[5,(3,0.0)],[6,(2,0.0)],[7,(1,0.0)],[8,(7,0.0)]]
    h_to_p = ['L','7','0/1','5/6','2','3','4','8','','','','']
    print('kala_sarpa',kala_sarpa(h_to_p))
    h_to_p = ['L','8','0/1','5/6','2','3','4','7','','','','']
    print('kala_sarpa',kala_sarpa(h_to_p))
    h_to_p = ['L','7/0','1','5','2','3','4','6/8','','','','']
    print('kala_sarpa',kala_sarpa(h_to_p))
    h_to_p = ['L','8/0','1','5','2','3','4','6/7','','','','']
    print('kala_sarpa',kala_sarpa(h_to_p))
    h_to_p = ['L','7','0/1','5','2','3','4','8','6','','','']
    print('kala_sarpa',kala_sarpa(h_to_p))
    h_to_p = ['L/0','7','1','5/6','2','3','4','8','','','','']
    print('kala_sarpa',kala_sarpa(h_to_p))
    h_to_p = ['L/6','5','8','','','','','','7','0/1','2/3','4']
    print('kala_sarpa',kala_sarpa(h_to_p))
    mrp = 'L'
    h_to_p = ['L/2','5','8','','','','','','7','0/1','6/3','4']
    print(h_to_p,'manglik',manglik(h_to_p,manglik_reference_planet=mrp))
    h_to_p = ['L','5/2','8','','','','','','7','0/1','6/3','4']
    print(h_to_p,'manglik',manglik(h_to_p,manglik_reference_planet=mrp))
    h_to_p = ['L','5','8','2','','','','','7','0/1','6/3','4']
    print(h_to_p,'manglik',manglik(h_to_p,manglik_reference_planet=mrp))
    h_to_p = ['L','5','8','','','','2','','7','0/1','6/3','4']
    print(h_to_p,'manglik',manglik(h_to_p,manglik_reference_planet=mrp))
    h_to_p = ['L','5','8','','','','','2','0/1','','6/3','4']
    print(h_to_p,'manglik',manglik(h_to_p,manglik_reference_planet=mrp))
    h_to_p = ['L','5','8','','','','','','0/1','','6/3','4/2']
    print(h_to_p,'manglik',manglik(h_to_p,manglik_reference_planet=mrp))
    print('Manglik Exception cases')
    h_to_p = ['5','L','8','','2','','','','0/1','','6/3','4']
    print(h_to_p,'manglik',manglik(h_to_p,manglik_reference_planet=mrp))
    h_to_p = ['','5','8','','L','','','','0/1','6/3','2','4']
    mng = manglik(h_to_p,manglik_reference_planet=mrp)
    print(h_to_p,'manglik',manglik(h_to_p,manglik_reference_planet=mrp))
    


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora import const, utils
from jhora.panchanga import drik
from jhora.horoscope.chart import charts, house
""" Paryaaya Dasa NOT IMPLEMENTED FULLY YET
    Dhasas are ok. Antardasa and periods are not matching JHora
"""
sidereal_year = const.sidereal_year
dhasa_adhipati_list = [0,4,6,10,0,4,6,10,0,4,6,10]
antardhasa_list = [6,0,8,10,4,8,6,0,8,10,4,8]

def _dhasa_duration(planet_positions,dhasa_lord):
    lord_owner = house.house_owner_from_planet_positions(planet_positions, dhasa_lord)
    house_of_lord = planet_positions[lord_owner+1][1][0]
    dhasa_period = (house_of_lord+13-dhasa_lord)%12
    if dhasa_lord in const.even_signs:
        dhasa_period = (dhasa_lord+13-house_of_lord)%12
    #print(dhasa_lord,lord_owner,house_of_lord,dhasa_period,dhasa_lord in const.even_signs)
    return dhasa_period
def _dhasa_lords(planet_positions,dhasa_seed):
    if dhasa_seed in const.dual_signs: # Dual and Chara Paryaaya
        ts = house.trines_of_the_raasi(dhasa_seed)
        sr = house.stronger_rasi_from_planet_positions(planet_positions, ts[0], ts[1])
        sr = house.stronger_rasi_from_planet_positions(planet_positions, sr, ts[2])
        dhasa_lords = [(sr+h-1)%12 for h in [1,5,9,2,6,10,3,7,11,4,8,12]]
        if sr in const.even_footed_signs:
            dhasa_lords = [(sr-h+13)%12 for h in [1,5,9,2,6,10,3,7,11,4,8,12]]
    elif dhasa_seed in const.movable_signs: # Movable and Ubhaya Paryaaya
        ts = house.quadrants_of_the_raasi(dhasa_seed)
        sr = house.stronger_rasi_from_planet_positions(planet_positions, ts[0], ts[1])
        sr = house.stronger_rasi_from_planet_positions(planet_positions, sr, ts[2])
        sr = house.stronger_rasi_from_planet_positions(planet_positions, sr, ts[3])
        dhasa_lords = [(sr+h-1)%12 for h in [1,4,7,10,2,5,8,11,3,6,9,12]]
        if sr in const.even_footed_signs: # Fixed and Sthira Paryaaya
            dhasa_lords = [(sr-h+13)%12 for h in [1,4,7,10,2,5,8,11,3,6,9,12]]
    else: #Fixed = Sthira Paryaaya
        sr = house.stronger_rasi_from_planet_positions(planet_positions, dhasa_seed, (dhasa_seed+6)%12)
        dhasa_lords = [(sr+h-1)%12 for h in [1,7,2,8,3,9,4,10,5,11,6,12]]
        if sr in const.even_footed_signs:
            dhasa_lords = [(sr-h+13)%12 for h in [1,7,2,8,3,9,4,10,5,11,6,12]]
    return dhasa_lords
def get_dhasa_antardhasa(dob,tob,place,divisional_chart_factor=6,years=1,months=1,sixty_hours=1,include_antardhasa=True,
                         use_tribhagi_variation=False):
    _dhasa_cycles = 2
    _tribhagi_factor = 1
    if use_tribhagi_variation:
        _tribhagi_factor=1./3.
        _dhasa_cycles = int(_dhasa_cycles/_tribhagi_factor)
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE, 
                                               divisional_chart_factor=divisional_chart_factor, years=years, 
                                               months=months, sixty_hours=sixty_hours)
    #print(planet_positions)
    asc_house = planet_positions[0][1][0]
    dhasa_seed = (asc_house+divisional_chart_factor-1)%12
    dhasa_lords = _dhasa_lords(planet_positions,dhasa_seed)
    dhasa_info = []
    start_jd = jd_at_dob
    for dc in range(_dhasa_cycles):
        for dhasa_lord in dhasa_lords:
            duration = round(_dhasa_duration(planet_positions,dhasa_lord)*_tribhagi_factor,2)
            bhukthis = _dhasa_lords(planet_positions, dhasa_lord)
            #print('bhukthis',bhukthis)
            if include_antardhasa:
                dd = duration/12
                for bhukthi_lord in bhukthis:
                    y,m,d,h = utils.jd_to_gregorian(start_jd)
                    dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                    dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,dd))
                    start_jd += dd * sidereal_year
            else:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,dhasa_start,duration))
                start_jd += duration * sidereal_year
    return dhasa_info
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests.paryaaya_dhasa_test()


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora import const, utils
from jhora.panchanga import drik
from jhora.horoscope.chart import charts, house
""" Navamsa Dasa """
sidereal_year = const.sidereal_year
"""
Birth in Sign    Ar    Ta    Ge    Cn    Le    Vi    Li    Sc    Sg    Cp    Aq    Pi
9 Years          Ar    Le    Li    Aq    Ar    Le    Li    Aq    Ar    Le    Li    Aq
                 0     4     6     10    0     4     6     10    0     4     6     10
                 Li    Ar    Sg    Aq    Le    Sg    Li    Ar    Sg    Aq    Le    Sg
                 6     0     8     10    4     8     6     0     8     10    4     8
"""
dhasa_adhipati_list = [0,4,6,10,0,4,6,10,0,4,6,10]
antardhasa_list = [6,0,8,10,4,8,6,0,8,10,4,8]
dhasa_duration = 9
def get_dhasa_antardhasa(dob,tob,place,divisional_chart_factor=9,years=1,months=1,sixty_hours=1,include_antardhasa=True):
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE, 
                                               divisional_chart_factor=divisional_chart_factor, years=years, 
                                               months=months, sixty_hours=sixty_hours)
    dhasa_seed = dhasa_adhipati_list[planet_positions[0][1][0]]
    dhasa_lords = [(dhasa_seed+h)%12 for h in range(12)]
    if dhasa_seed in const.even_signs:
        dhasa_lords = [(dhasa_seed+6-h+12)%12 for h in range(12)]
    dhasa_info = []
    start_jd = jd_at_dob
    for dhasa_lord in dhasa_lords:
        duration = dhasa_duration
        bukthi_seed = antardhasa_list[dhasa_lord]
        #print('dhasa_lord',house.rasi_names_en[dhasa_lord],'bukthi seed',house.rasi_names_en[bukthi_seed])
        bhukthis = [(bukthi_seed+h)%12 for h in range(12)]
        #print('bhukthis',bhukthis)
        if include_antardhasa:
            dd = duration/12
            for bhukthi_lord in bhukthis:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,dd))
                start_jd += dd * sidereal_year
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((dhasa_lord,dhasa_start,duration))
            start_jd += duration * sidereal_year
    return dhasa_info
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests.navamsa_dhasa_test()


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from PyQt6 import QtWidgets
from PyQt6.QtCore import Qt
from PyQt6.QtWidgets import QWidget, QGridLayout, QComboBox
from PyQt6.QtGui import QCursor
import pyqtgraph as pg
from jhora import utils, const
from jhora.panchanga import drik

""" mars(2),mercury(3),jupiter(4),venus(5),saturn(6),uranus(9), neptune(10), pluto(11), 
    earth(-1). Ceres (99) added just for reference and not to be used. """
_starting_planet_index = 2
retrogression_planets_periods = {2:(686.97959,18),3:(87.96926,9),4:(4332.8201,12),5:(224.7008,8),6:(10755.699,59),
                                 9:(20687.153,60),10:(60190.03,165),11:(90582.03,248),99:(1680.22107,50),-1:(365.25636,1)}
_track_mouse_coordinates = False
dist = lambda T : T**(2/3) # Kepler
from numpy import linspace, sin, cos, pi
def get_planet_entry_data(start_jd,place):
    npe_data = {}
    for planet in list(retrogression_planets_periods.keys())[:5]:
        npe_data[planet]={}
        jd_utc = start_jd - place.timezone/24.
        p_long = drik.sidereal_longitude(jd_utc, planet)
        y,m,d,_ = utils.jd_to_gregorian(start_jd)
        p_date = drik.Date(y,m,d)
        rasi = p_long//30
        theta = linspace(0, 2*pi, 13)
        for i,t in enumerate(theta[:-1]):
            r = int(rasi+i)%12
            npe = drik.next_planet_entry_date(planet, p_date, place,raasi=None,direction=1)
            y,m,d,_ = utils.jd_to_gregorian(npe[0]); p_long = npe[1]
            npe_data[planet][r]=((y,m,d),p_long)
            p_date = drik.Date(y,m,d)
            #print(planet,'next raasi',p_date,p_long,p_long//30)
    return npe_data 
def get_retrogression_orbit_data(planet,start_angle=0):
    #print('planet',planet,retrogression_planets_periods[planet],'periods',periods)
    periods = retrogression_planets_periods[planet][1]
    T1 = retrogression_planets_periods[planet][0]
    T2 = retrogression_planets_periods[-1][0]
    d1 = dist(T1)
    d2 = dist(T2)

    theta = linspace(start_angle, 2*pi*periods+start_angle, 1000)
    x = d1*cos(T2*theta/T1) - d2*cos(theta)
    y = d1*sin(T2*theta/T1) - d2*sin(theta)
    return x,y
class VakraGathiPlot(QtWidgets.QMainWindow):
    def __init__(self,start_jd=None,place=None,planet=6,color=(255,0,255),axis_off=True,title='',plot_width=500,plot_height=500,
                 language='Tamil'):
        super().__init__()
        self._start_jd = start_jd
        self._place = place
        self.title = title
        self.color = color
        self.plot_width = plot_width; self.plot_height = plot_height; self.axis_off = axis_off
        self._next_planet_entry_data = {}
        if self._start_jd != None and self._place != None:
            self._next_planet_entry_data = get_planet_entry_data(start_jd,place)
        # Temperature vs time plot
        self.plot_graph = pg.PlotWidget()
        widget = QWidget()
        layout = QGridLayout()
        self._planet_combo = QComboBox()
        _planet_list = utils.PLANET_NAMES[2:7]+utils.PLANET_NAMES[9:12]
        self._planet_combo.addItems(_planet_list)
        self._planet_combo.currentIndexChanged.connect(self._update_chart)
        self._planet_combo.setCurrentIndex(planet-_starting_planet_index)
        layout.addWidget(self._planet_combo,0,0)
        self._lang_combo = QComboBox()
        self._lang_combo.addItems(const.available_languages.keys())
        self._lang_combo.setCurrentText(language)
        self._language = const.available_languages[language]
        self._lang_combo.activated.connect(self._change_language)
        layout.addWidget(self._lang_combo,1,0)
        layout.addWidget(self.plot_graph,0,1,3,1)
        widget.setLayout(layout)
        self._change_language()
        self.setCentralWidget(widget)
    def _change_language(self):
        _language_index = self._lang_combo.currentIndex()
        self._language = list(const.available_languages.values())[_language_index]
        #print('self._language',self._language)
        utils.set_language(self._language)
        msgs = utils.resource_strings
        self._lang_combo.clear()
        self._lang_combo.addItems([msgs[l.lower()+'_str'] for l in const.available_languages.keys()])
        self._lang_combo.setCurrentIndex(_language_index)

        _planet_list = utils.PLANET_NAMES[2:7]+utils.PLANET_NAMES[9:12]
        _current_planet_index = self._planet_combo.currentIndex()
        self._planet_combo.clear()
        self._planet_combo.addItems(_planet_list)
        self._planet_combo.setCurrentIndex(_current_planet_index)
    def onMouseMoved(self, evt):
        #print('mouse moved',evt.x(),evt.y())
        if self.plot_graph.plotItem.vb.mapSceneToView(evt) and self.line.cursor()==Qt.CursorShape.PointingHandCursor:
            point =self.plot_graph.plotItem.vb.mapSceneToView(evt)
            print(point.x(),point.y())
            #self.mouse_label.setPos(-560,560)
            self.mouse_label.setText("Date： {0} \nLongitude: {1}".format(point.x(), point.y()),color='b')
    def _update_chart(self):
        self.plot_graph.clear()
        planet = self._planet_combo.currentIndex()+_starting_planet_index
        if self._planet_combo.currentIndex()>4: planet = self._planet_combo.currentIndex()+_starting_planet_index+2
        if planet not in retrogression_planets_periods.keys(): return
        # Draw retrograde graph
        self.title = utils.resource_strings['retrograde_as_seen_from_earth_str_1']+' '+utils.PLANET_NAMES[planet]+' '+utils.resource_strings['retrograde_as_seen_from_earth_str_2']
        self.plot_graph.setBackground("w")
        pen = pg.mkPen(color=self.color)
        self.plot_graph.setTitle(self.title,color=(0,0,0))
        self.plot_graph.setMinimumSize(self.plot_width,self.plot_height)
        T1 = retrogression_planets_periods[planet][0]; T2 = retrogression_planets_periods[-1][0]
        d1 = dist(T1); d2 = dist(T2)
        label_factor = 0.85
        if self.axis_off:
            self.plot_graph.getPlotItem().hideAxis('left');self.plot_graph.getPlotItem().hideAxis('bottom')
        p_long = 0
        if self._start_jd != None and self._place != None:
            jd_utc = self._start_jd - self._place.timezone/24.
            p_long = drik.sidereal_longitude(jd_utc, planet)
            y,m,d,_ = utils.jd_to_gregorian(self._start_jd)
            date_str = str(y)+'-'+str(m)+'-'+str(d)
            lbl = pg.TextItem(date_str,color=(0,0,0))
            self.plot_graph.addItem(lbl)
            th = p_long*pi/180.0
            x_label = (d1+d2)*cos(th)*0.5; y_label = (d1+d2)*sin(th)*0.5
            lbl.setPos(x_label,y_label)
            # Draw DOB Line
            x_line = x_label; y_line = y_label
            line = pg.LineROI(pos1=(0,0),pos2=(x_line,y_line),width=1,pen = pg.mkPen(color=(0,0,255)),movable=False)
            self.plot_graph.addItem(line)
        #print(utils.PLANET_NAMES[planet],utils.to_dms(p_long,is_lat_long='plong'))
        x,y=get_retrogression_orbit_data(planet,p_long)
        self.line = self.plot_graph.plot(x, y, pen=pen)
        self.line.setCursor(QCursor(Qt.CursorShape.PointingHandCursor))
        if _track_mouse_coordinates:
            #track mouse coordinates 
            self.mouse_label = pg.TextItem(text="Date: {} \nLongitude: {}".format(0, 0))
            self.mouse_label.setPos(-560,560)
            self.plot_graph.addItem(self.mouse_label)
            self.setMouseTracking(True)
            self.plot_graph.scene().sigMouseMoved.connect(self.onMouseMoved)
        # Write Earth Label
        labels = utils.RAASI_LIST+[utils.RAASI_LIST[0]]
        lbl = pg.TextItem(utils.resource_strings['earth_str'],color=(0,0,0))
        self.plot_graph.addItem(lbl); lbl.setPos(0.0,0.0)
        y,m,d,_ = utils.jd_to_gregorian(self._start_jd)
        p_date = drik.Date(y,m,d)
        raasi = p_long//30
        theta = linspace(0, 2*pi, 13)
        for i,t in enumerate(theta[:-1]):
            r = i;date_str = ''
            if planet <=6:
                r = int(raasi+i-1)%12
                #npe = drik.next_planet_entry_date(planet, p_date, place,raasi=None,direction=1)
                npe = self._next_planet_entry_data[planet][r]
                y,m,d = npe[0]; p_long = utils.to_dms(npe[1],is_lat_long='plong')
                date_str = str(y)+'-'+str(m)+'-'+str(d)
                p_date = drik.Date(y,m,d)
            # Write Raasi Label
            lbl = pg.TextItem(labels[r]+'\n'+date_str,color=(0,0,0))
            self.plot_graph.addItem(lbl)
            th = 0.5*(theta[r]+theta[r+1])
            x_label = (d1+d2)*cos(th)*label_factor; y_label = (d1+d2)*sin(th)*label_factor
            lbl.setPos(x_label,y_label)
            # Draw Raasi Separation Line
            x_line = (d1+d2)*cos(t)*label_factor; y_line = (d1+d2)*sin(t)*label_factor
            line = pg.LineROI(pos1=(0,0),pos2=(x_line,y_line),width=0.5,pen = pg.mkPen(color=(255,0,0)),movable=False)
            self.plot_graph.addItem(line)
            line.setPos(0,0)
        
import sys
from _datetime import datetime
from jhora import utils
from jhora.panchanga import drik
#const._TROPICAL_MODE = False
utils.set_language('ta')
app = QtWidgets.QApplication(sys.argv)
planet = 9 # 2 = Mars,3=Merc,4-Jup,5=Ven,6=Sat,7=Ura,8=Nep,9=Plu
current_date_str,current_time_str = datetime.now().strftime('%Y,%m,%d;%H:%M:%S').split(';')
dob = tuple(map(int,current_date_str.split(',')))
tob = tuple(map(int,current_time_str.split(':')))
loc = utils.get_place_from_user_ip_address(); place = drik.Place(loc[0],loc[1],loc[2],loc[3])
jd = utils.julian_day_number(dob, tob)
"""
npe = get_planet_entry_data(jd,place)
print(npe)
exit()
"""
w = VakraGathiPlot(jd,place,planet=planet)
w.show()
app.exec()




import os
import sys
path_root = os.path.dirname(os.path.abspath('../'))
if path_root not in sys.path:
    sys.path.append(str(path_root))
    print('panchanga',path_root,'added to system path',sys.path)